Linux  R2.6.5-7.282-sn2 FORTRAN90/SX         Rev.360        Mon Oct  1 21:50:41 2012
FILE NAME: rad_Earth_V2.f90
PROGRAM NAME: rad_earth_v2
DIAGNOSTIC LIST

  LINE  LEVEL( NO.): DIAGNOSTIC MESSAGE

   168  vec  (   3): Unvectorized loop.
   168  vec  (  13): Overhead of loop division is too large.
   169  opt  (1592): Outer loop unrolled inside inner loop.
   169  vec  (   4): Vectorized array expression.
   169  vec  (   4): Vectorized array expression.
   504  opt  (1592): Outer loop unrolled inside inner loop.
   504  vec  (   4): Vectorized array expression.
   504  vec  (   4): Vectorized array expression.
   517  opt  (  11): Fused array assignments. :line 517 - 521
   517  vec  (   4): Vectorized array expression.
Linux  R2.6.5-7.282-sn2 FORTRAN90/SX         Rev.360        Mon Oct  1 21:50:41 2012
FILE NAME: rad_Earth_V2.f90
PROGRAM NAME: rad_earth_v2
TRANSFORMATION LIST

  LINE                   FORTRAN STATEMENT

     1  != ϵ絤ͥǥ Ver. 2
     2  !
     3  != radiation model for the Earth's atmosphere Ver. 2
     4  !
     5  ! Authors::   Yoshiyuki O. Takahashi
     6  ! Version::   $Id: rad_Earth_V2.f90,v 1.6 2012-09-08 15:25:05 yot Exp $
     7  ! Tag Name::  $Name: dcpam5-20120921 $
     8  ! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
     9  ! License::   See COPYRIGHT[link:../../../COPYRIGHT]
    10  !
    11  module rad_Earth_V2
    12    !
    13    != ϵ絤ͥǥ Ver. 2
    14    !
    15    != radiation model for the Earth's atmosphere Ver. 1
    16    !
    17    ! <b>Note that Japanese and English are described in parallel.</b>
    18    !
    19    ! ϵ絤ͥǥ.
    20    !
    21    ! This is a radiation model for the Earth's atmospehre.
    22    !
    23    ! Radiation in the wavenumber range from    0 to  3000 cm-1
    24    ! is calculated in the routine of long wave radiation.
    25    ! Radiation in the wavenumber range from 1000 to 57143 cm-1 (0.175 to 10 micron)
    26    ! is calculated in the routine of shortwave radiation.
    27    !
    28    !== References
    29    !
    30    !
    31    !== Procedures List
    32    !
    33    ! RadEarthV2Flux :: ͥեåη׻
    34    ! ------------          :: ------------
    35    ! RadEarthV2Flux :: Calculate radiation flux
    36    !
    37    !== NAMELIST
    38    !
    39    ! NAMELIST#rad_Earth_V2_nml
    40    !
    41  
    42    ! USE statements
    43    !
    44  
    45    !
    46    ! Kind type parameter
    47    !
    48    use dc_types, only: DP, &      ! Double precision.
    49      &                 STRING, &  ! Strings.
    50      &                 TOKEN      ! Keywords.
    51  
    52    ! å
    53    ! Message output
    54    !
    55    use dc_message, only: MessageNotify
    56  
    57    !
    58    ! Physical constants settings
    59    !
    60    use constants, only: Grav     ! $ g $ [m s-2].
    61                                  !
    62                                  ! Gravitational acceleration
    63  
    64    ! ʻ
    65    ! Grid points settings
    66    !
    67    use gridset, only: imax, & ! ٳʻ.
    68                               ! Number of grid points in longitude
    69      &                jmax, & ! ٳʻ.
    70                               ! Number of grid points in latitude
    71      &                kmax    ! ľؿ.
    72                               ! Number of vertical level
    73  
    74    implicit none
    75  
    76    private
    77  
    78    real(DP), save :: VMRCO2            ! Volume mixing ratio of CO2
    79    real(DP), save :: MeanMolWeight
    80    real(DP), save :: H2OMolWeight
    81    real(DP), save :: CO2MolWeight
    82  
    83    real(DP), allocatable, save :: xyz_QO3(:,:,:)
    84                                ! O3 ʬ (1)
    85                                ! O3 distribution (1)
    86  
    87  !!$  integer , save              :: SWVer
    88  !!$  integer , save              :: LWVer
    89  
    90    ! ѿ
    91    ! Public variables
    92    !
    93    logical, save :: rad_Earth_V2_inited = .false.
    94                                ! ե饰.
    95                                ! Initialization flag
    96  
    97    public :: RadEarthV2Flux
    98  !!$  public :: RadEarthV2FluxforNHM
    99    public :: RadEarthV2Init
   100  
   101    character(*), parameter:: module_name = 'rad_Earth_V2'
   102                                ! ⥸塼̾.
   103                                ! Module name
   104    character(*), parameter:: version = &
   105      & '$Name: dcpam5-20120921 $' // &
   106      & '$Id: rad_Earth_V2.f90,v 1.6 2012-09-08 15:25:05 yot Exp $'
   107                                ! ⥸塼ΥС
   108                                ! Module version
   109  
   110    !--------------------------------------------------------------------------------------
   111  
   112  contains
   113  
   114    !--------------------------------------------------------------------------------------
   115  
   116    subroutine RadEarthV2Flux(                                                 &
   117      & xy_SurfAlbedo,                                                         & ! (in)
   118      & xyz_Press, xyr_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, & ! (in)
   119      & xyz_CloudCover,                                                        & ! (in)
   120      & xy_SurfTemp,                                                           & ! (in)
   121      & xyr_RadSUwFlux, xyr_RadSDwFlux,                                        & ! (out)
   122      & xyr_RadLUwFlux, xyr_RadLDwFlux,                                        & ! (out)
   123      & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                                 & ! (out)
   124      & )
   125  
   126  
   127      ! USE statements
   128      !
   129  
   130      real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   131      real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   132      real(DP), intent(in ) :: xyr_Press       (0:imax-1, 1:jmax, 0:kmax)
   133      real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   134      real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   135                                ! $ q $ .   . Mass mixing ratio of constituents (1)
   136      real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   137      real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
   138      real(DP), intent(in ) :: xyz_CloudCover  (0:imax-1, 1:jmax, 1:kmax)
   139      real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   140      real(DP), intent(out) :: xyr_RadSUwFlux   (0:imax-1, 1:jmax, 0:kmax)
   141      real(DP), intent(out) :: xyr_RadSDwFlux   (0:imax-1, 1:jmax, 0:kmax)
   142      real(DP), intent(out) :: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   143      real(DP), intent(out) :: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   144      real(DP), intent(out) :: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   145      real(DP), intent(out) :: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   146  
   147  !!$    real(DP):: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   148  !!$    real(DP):: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   149  !!$    real(DP):: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   150  !!$    real(DP):: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   151  
   152  
   153      ! Work variables
   154      !
   155      real(DP) :: xyz_DelAtmMass   (0:imax-1, 1:jmax, 1:kmax)
   156  
   157      integer  :: k
   158  
   159  
   160      ! ǧ
   161      ! Initialization check
   162      !
   163      if ( .not. rad_Earth_V2_inited ) then
   164        call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   165      end if
   166  
   167  
   168      do k = 1, kmax
   169        xyz_DelAtmMass(:,:,k) = ( xyr_Press(:,:,k-1) - xyr_Press(:,:,k ) ) / Grav
     .        if (jmax .gt. 0) then                                             
     .           J1 = and(jmax,3)                                               
     .  !CDIR    NODEP                                                          
     .           do t231 = 1, J1                                                
     .              D1 = 1.D0/grav                                              
     .  !CDIR       NODEP                                                       
     .              do t233 = 1, imax                                           
     .                 xyz_delatmmass(t233-1,t231,k) = (xyr_press(t233-1,t231,k-
     .       1            1)-xyr_press(t233-1,t231,k))*D1                       
     .              end do                                                      
     .           end do                                                         
     .  !CDIR    NODEP                                                          
     .           do t231 = J1 + 1, jmax, 4                                      
     .              D2 = 1.D0/grav                                              
     .              D3 = 1.D0/grav                                              
     .              D4 = 1.D0/grav                                              
     .              D5 = 1.D0/grav                                              
     .  !CDIR       NODEP                                                       
     .              do t233 = 1, imax                                           
     .                 xyz_delatmmass(t233-1,t231,k) = (xyr_press(t233-1,t231,k-
     .       1            1)-xyr_press(t233-1,t231,k))*D2                       
     .                 xyz_delatmmass(t233-1,t231+1,k) = (xyr_press(t233-1,t231+
     .       1            1,k-1)-xyr_press(t233-1,t231+1,k))*D3                 
     .                 xyz_delatmmass(t233-1,t231+2,k) = (xyr_press(t233-1,t231+
     .       1            2,k-1)-xyr_press(t233-1,t231+2,k))*D4                 
     .                 xyz_delatmmass(t233-1,t231+3,k) = (xyr_press(t233-1,t231+
     .       1            3,k-1)-xyr_press(t233-1,t231+3,k))*D5                 
     .              end do                                                      
     .           end do                                                         
     .        endif                                                             
   170      end do
   171  
   172      call RadEarthV2FluxCore(                                                        &
   173        & xy_SurfAlbedo,                                                              &
   174        & xyz_DelAtmMass, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   175        & xyz_CloudCover,                                                             &
   176        & xy_SurfTemp,                                                                &
   177        & xyr_RadSUwFlux, xyr_RadSDwFlux,                                             &
   178        & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
   179        & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
   180        & )
   181  
   182  
   183    end subroutine RadEarthV2Flux
   184  
   185    !--------------------------------------------------------------------------------------
   186  !!$
   187  !!$  subroutine RadEarthV2FluxforNHM(                                          &
   188  !!$    & r_Height, xy_SurfAlbedo,                                              &
   189  !!$    & xyz_Dens, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   190  !!$    & xy_SurfTemp,                                                          &
   191  !!$    & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                          &
   192  !!$    & )
   193  !!$
   194  !!$
   195  !!$    ! USE statements
   196  !!$    !
   197  !!$
   198  !!$    real(DP), intent(in ) :: r_Height                          (0:kmax)
   199  !!$    real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   200  !!$    real(DP), intent(in ) :: xyz_Dens        (0:imax-1, 1:jmax, 1:kmax)
   201  !!$    real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   202  !!$    real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   203  !!$    real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   204  !!$                              ! $ q $ .   . Mass mixing ratio of constituents (1)
   205  !!$    real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   206  !!$    real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
   207  !!$    real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   208  !!$    real(DP), intent(out) :: xyr_RadSFlux    (0:imax-1, 1:jmax, 0:kmax)
   209  !!$    real(DP), intent(out) :: xyr_RadLFlux    (0:imax-1, 1:jmax, 0:kmax)
   210  !!$    real(DP), intent(out) :: xyra_DelRadLFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   211  !!$
   212  !!$
   213  !!$    ! Work variables
   214  !!$    !
   215  !!$    real(DP) :: xyz_DelAtmMass   (0:imax-1, 1:jmax, 1:kmax)
   216  !!$
   217  !!$    integer  :: k
   218  !!$
   219  !!$
   220  !!$    ! ǧ
   221  !!$    ! Initialization check
   222  !!$    !
   223  !!$    if ( .not. rad_Earth_V2_inited ) then
   224  !!$      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   225  !!$    end if
   226  !!$
   227  !!$
   228  !!$    do k = 1, kmax
   229  !!$      xyz_DelAtmMass(:,:,k) = xyz_Dens(:,:,k) * ( r_Height(k) - r_Height(k-1) )
   230  !!$    end do
   231  !!$
   232  !!$    call RadEarthV2FluxCore(                                                        &
   233  !!$      & xy_SurfAlbedo,                                                              &
   234  !!$      & xyz_DelAtmMass, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   235  !!$      & xy_SurfTemp,                                                                &
   236  !!$      & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                                &
   237  !!$      & )
   238  !!$
   239  !!$
   240  !!$  end subroutine RadEarthV2FluxforNHM
   241  !!$
   242    !--------------------------------------------------------------------------------------
   243  !!$
   244  !!$  subroutine RadDcpamEV2FluxforNHM2DWrapper(                    &
   245  !!$    & r_Height, x_SurfAlbedo,                                   &
   246  !!$    & xz_Dens, xz_Press, xz_Temp, xz_QH2OVap, xz_QH2OLiq,       &
   247  !!$    & x_SurfTemp,                                               &
   248  !!$    & xr_RadSFlux, xr_RadLFlux, xra_DelRadLFlux                 &
   249  !!$    & )
   250  !!$
   251  !!$
   252  !!$    ! USE statements
   253  !!$    !
   254  !!$
   255  !!$
   256  !!$    real(DP), intent(in ) :: r_Height                 (0:kmax)
   257  !!$    real(DP), intent(in ) :: x_SurfAlbedo   (0:imax-1)
   258  !!$    real(DP), intent(in ) :: xz_Dens        (0:imax-1, 1:kmax)
   259  !!$    real(DP), intent(in ) :: xz_Press       (0:imax-1, 1:kmax)
   260  !!$    real(DP), intent(in ) :: xz_Temp        (0:imax-1, 1:kmax)
   261  !!$    real(DP), intent(in ) :: xz_QH2OVap     (0:imax-1, 1:kmax)
   262  !!$    real(DP), intent(in ) :: xz_QH2OLiq     (0:imax-1, 1:kmax)
   263  !!$    real(DP), intent(in ) :: x_SurfTemp     (0:imax-1)
   264  !!$    real(DP), intent(out) :: xr_RadSFlux    (0:imax-1, 0:kmax)
   265  !!$    real(DP), intent(out) :: xr_RadLFlux    (0:imax-1, 0:kmax)
   266  !!$    real(DP), intent(out) :: xra_DelRadLFlux(0:imax-1, 0:kmax, 0:1)
   267  !!$
   268  !!$
   269  !!$    ! Work variables
   270  !!$    !
   271  !!$    real(DP) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   272  !!$    real(DP) :: xyz_Dens        (0:imax-1, 1:jmax, 1:kmax)
   273  !!$    real(DP) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   274  !!$    real(DP) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   275  !!$    real(DP) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   276  !!$    real(DP) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   277  !!$    real(DP) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   278  !!$    real(DP) :: xyr_RadSFlux    (0:imax-1, 1:jmax, 0:kmax)
   279  !!$    real(DP) :: xyr_RadLFlux    (0:imax-1, 1:jmax, 0:kmax)
   280  !!$    real(DP) :: xyra_DelRadLFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   281  !!$
   282  !!$    integer  :: k
   283  !!$
   284  !!$
   285  !!$    ! ǧ
   286  !!$    ! Initialization check
   287  !!$    !
   288  !!$    if ( .not. rad_dcpam_E_V2_inited ) then
   289  !!$      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   290  !!$    end if
   291  !!$
   292  !!$
   293  !!$    if ( jmax /= 1 ) then
   294  !!$      call MessageNotify( 'E', module_name, 'jmax must be 1.' )
   295  !!$    end if
   296  !!$
   297  !!$
   298  !!$    xy_SurfAlbedo   (:, 1)       = x_SurfAlbedo   (:)
   299  !!$    do k = 1, kmax
   300  !!$      xyz_Dens        (:, 1, k)    = xz_Dens        (:, k)
   301  !!$      xyz_Press       (:, 1, k)    = xz_Press       (:, k)
   302  !!$      xyz_Temp        (:, 1, k)    = xz_Temp        (:, k)
   303  !!$      xyz_QH2OVap     (:, 1, k)    = xz_QH2OVap     (:, k)
   304  !!$      xyz_QH2OLiq     (:, 1, k)    = xz_QH2OLiq     (:, k)
   305  !!$    end do
   306  !!$    xy_SurfTemp     (:, 1)       = x_SurfTemp     (:)
   307  !!$
   308  !!$    call RadDcpamEV2FluxforNHM(                                       &
   309  !!$      & r_Height, xy_SurfAlbedo,                                      &
   310  !!$      & xyz_Dens, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq,      &
   311  !!$      & xy_SurfTemp,                                                  &
   312  !!$      & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                  &
   313  !!$      & )
   314  !!$
   315  !!$    xr_RadSFlux    (:, :)    = xyr_RadSFlux    (:, 1, :)
   316  !!$    xr_RadLFlux    (:, :)    = xyr_RadLFlux    (:, 1, :)
   317  !!$    xra_DelRadLFlux(:, :, :) = xyra_DelRadLFlux(:, 1, :, :)
   318  !!$
   319  !!$
   320  !!$  end subroutine RadDcpamEV2FluxforNHM2DWrapper
   321  !!$
   322    !--------------------------------------------------------------------------------------
   323  !!$
   324  !!$  subroutine RadDcpamEV2Flux2DWrapper(                         &
   325  !!$    & x_SurfAlbedo,                                            &
   326  !!$    & xz_Press, xr_Press, xz_Temp, xz_QH2OVap, xz_QH2OLiq,     &
   327  !!$    & x_SurfTemp,                                              &
   328  !!$    & xr_RadSFlux, xr_RadLFlux, xra_DelRadLFlux                &
   329  !!$    & )
   330  !!$
   331  !!$
   332  !!$    ! USE statements
   333  !!$    !
   334  !!$
   335  !!$
   336  !!$
   337  !!$    real(DP), intent(in ) :: x_SurfAlbedo   (0:imax-1)
   338  !!$    real(DP), intent(in ) :: xz_Press       (0:imax-1, 1:kmax)
   339  !!$    real(DP), intent(in ) :: xr_Press       (0:imax-1, 0:kmax)
   340  !!$    real(DP), intent(in ) :: xz_Temp        (0:imax-1, 1:kmax)
   341  !!$    real(DP), intent(in ) :: xz_QH2OVap     (0:imax-1, 1:kmax)
   342  !!$    real(DP), intent(in ) :: xz_QH2OLiq     (0:imax-1, 1:kmax)
   343  !!$    real(DP), intent(in ) :: x_SurfTemp     (0:imax-1)
   344  !!$    real(DP), intent(out) :: xr_RadSFlux    (0:imax-1, 0:kmax)
   345  !!$    real(DP), intent(out) :: xr_RadLFlux    (0:imax-1, 0:kmax)
   346  !!$    real(DP), intent(out) :: xra_DelRadLFlux(0:imax-1, 0:kmax, 0:1)
   347  !!$
   348  !!$
   349  !!$    ! Work variables
   350  !!$    !
   351  !!$    real(DP) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   352  !!$    real(DP) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   353  !!$    real(DP) :: xyr_Press       (0:imax-1, 1:jmax, 0:kmax)
   354  !!$    real(DP) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   355  !!$    real(DP) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   356  !!$    real(DP) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   357  !!$    real(DP) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   358  !!$    real(DP) :: xyr_RadSFlux    (0:imax-1, 1:jmax, 0:kmax)
   359  !!$    real(DP) :: xyr_RadLFlux    (0:imax-1, 1:jmax, 0:kmax)
   360  !!$    real(DP) :: xyra_DelRadLFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   361  !!$
   362  !!$
   363  !!$    ! ǧ
   364  !!$    ! Initialization check
   365  !!$    !
   366  !!$    if ( .not. rad_dcpam_E_V2_inited ) then
   367  !!$      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   368  !!$    end if
   369  !!$
   370  !!$
   371  !!$    if ( jmax /= 1 ) then
   372  !!$      call MessageNotify( 'E', module_name, 'jmax must be 1.' )
   373  !!$    end if
   374  !!$
   375  !!$
   376  !!$    xy_SurfAlbedo   (:, 1)       = x_SurfAlbedo   (:)
   377  !!$    xyz_Press       (:, 1, :)    = xz_Press       (:, :)
   378  !!$    xyr_Press       (:, 1, :)    = xr_Press       (:, :)
   379  !!$    xyz_Temp        (:, 1, :)    = xz_Temp        (:, :)
   380  !!$    xyz_QH2OVap     (:, 1, :)    = xz_QH2OVap     (:, :)
   381  !!$    xyz_QH2OLiq     (:, 1, :)    = xz_QH2OLiq     (:, :)
   382  !!$    xy_SurfTemp     (:, 1)       = x_SurfTemp     (:)
   383  !!$
   384  !!$    call RadDcpamEV2Flux(                                             &
   385  !!$      & xy_SurfAlbedo,                                                &
   386  !!$      & xyz_Press, xyr_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq,     &
   387  !!$      & xy_SurfTemp,                                                  &
   388  !!$      & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                  &
   389  !!$      & )
   390  !!$
   391  !!$    xr_RadSFlux    (:, :)    = xyr_RadSFlux    (:, 1, :)
   392  !!$    xr_RadLFlux    (:, :)    = xyr_RadLFlux    (:, 1, :)
   393  !!$    xra_DelRadLFlux(:, :, :) = xyra_DelRadLFlux(:, 1, :, :)
   394  !!$
   395  !!$
   396  !!$  end subroutine RadDcpamEV2Flux2DWrapper
   397  !!$
   398    !--------------------------------------------------------------------------------------
   399  
   400    subroutine RadEarthV2FluxCore(                                                  &
   401      & xy_SurfAlbedo,                                                              &
   402      & xyz_DelAtmMass, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   403      & xyz_CloudCover,                                                             &
   404      & xy_SurfTemp,                                                                &
   405      & xyr_RadSUwFlux, xyr_RadSDwFlux,                                             &
   406      & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
   407      & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
   408      & )
   409  
   410  
   411      ! USE statements
   412      !
   413  
   414      ! ҥȥǡ
   415      ! History data output
   416      !
   417      use gtool_historyauto, only: HistoryAutoPut
   418  
   419      ! 
   420      ! Time control
   421      !
   422      use timeset, only: &
   423        & TimeN                 ! ƥå $ t $ λ.
   424                                ! Time of step $ t $.
   425  
   426      ! O3 ʬۤ
   427      ! Set O3 distribution
   428      !
   429      use set_o3, only: SetO3
   430  
   431  !!$    ! dcpam ϵ絤ûͥǥ Ver. 2
   432  !!$    ! dcpam short wave radiation model for the Earth's atmosphere Ver. 2
   433  !!$    !
   434  !!$    use radiation_dcpam_E_SW_V2, only: RadiationDcpamESWV2Flux
   435  
   436      ! ϵ絤ûͥǥ Ver. 2.1
   437      ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   438      !
   439      use rad_Earth_SW_V2_1, only: RadEarthSWV21Flux
   440  !!$
   441  !!$    ! ϵ絤ûͥǥ Ver. 2.2
   442  !!$    ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   443  !!$    !
   444  !!$    use rad_Earth_SW_V2_2, only: RadEarthSWV22Flux
   445  
   446  !!$    ! dcpam ϵ絤Ĺͥǥ Ver. 2
   447  !!$    ! dcpam long wave radiation model for the Earth's atmosphere Ver. 2
   448  !!$    !
   449  !!$    use radiation_dcpam_E_LW_V2, only : RadiationDcpamELWV2Flux
   450  !!$
   451  !!$    ! dcpam ϵ絤Ĺͥǥ Ver. 2.1
   452  !!$    ! dcpam long wave radiation model for the Earth's atmosphere Ver. 2.1
   453  !!$    !
   454  !!$    use radiation_dcpam_E_LW_V2_1, only : RadiationDcpamELWV21Flux
   455  !!$
   456  !!$    ! dcpam ϵ絤Ĺͥǥ Ver. 2.2
   457  !!$    ! dcpam long wave radiation model for the Earth's atmosphere Ver. 2.2
   458  !!$    !
   459  !!$    use radiation_dcpam_E_LW_V2_2, only : RadiationDcpamELWV22Flux
   460  
   461      ! ϵ絤Ĺͥǥ Ver. 2.3
   462      ! long wave radiation model for the Earth's atmosphere Ver. 2.3
   463      !
   464      use rad_Earth_LW_V2_3, only : RadEarthLWV23Flux
   465  
   466      real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   467      real(DP), intent(in ) :: xyz_DelAtmMass  (0:imax-1, 1:jmax, 1:kmax)
   468      real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   469      real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   470      real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   471                                ! $ q $ .   . Mass mixing ratio of constituents (1)
   472      real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   473      real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
   474      real(DP), intent(in ) :: xyz_CloudCover  (0:imax-1, 1:jmax, 1:kmax)
   475      real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   476      real(DP), intent(out) :: xyr_RadSUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   477      real(DP), intent(out) :: xyr_RadSDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   478      real(DP), intent(out) :: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   479      real(DP), intent(out) :: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   480      real(DP), intent(out) :: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   481      real(DP), intent(out) :: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   482  
   483  
   484      ! Work variables
   485      !
   486      real(DP) :: xyz_QCO2         (0:imax-1, 1:jmax, 1:kmax)
   487  
   488      real(DP) :: xyz_DelCO2Mass   (0:imax-1, 1:jmax, 1:kmax)
   489      real(DP) :: xyz_DelH2OVapMass(0:imax-1, 1:jmax, 1:kmax)
   490      real(DP) :: xyz_DelH2OLiqMass(0:imax-1, 1:jmax, 1:kmax)
   491      real(DP) :: xyz_DelH2OSolMass(0:imax-1, 1:jmax, 1:kmax)
   492      real(DP) :: xyz_DelO3Mass    (0:imax-1, 1:jmax, 1:kmax)
   493  
   494  
   495  
   496      ! ǧ
   497      ! Initialization check
   498      !
   499      if ( .not. rad_Earth_V2_inited ) then
   500        call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   501      end if
   502  
   503  
   504      xyz_QCO2 = VMRCO2 * CO2MolWeight / MeanMolWeight
     .        if (xyz_qco2.DSC.U2 .gt. 0) then                                  
     .           J1 = and(xyz_qco2.DSC.U2,3)                                    
     .  !CDIR    NODEP                                                          
     .           do t338 = 1, J1                                                
     .              D1 = vmrco2/meanmolweight                                   
     .  !CDIR       NODEP                                                       
     .              do t340 = 1, xyz_qco2.DSC.U1 + 1                            
     .                 xyz_qco2(t340-1,t338,t336+1) = co2molweight*D1           
     .              end do                                                      
     .           end do                                                         
     .  !CDIR    NODEP                                                          
     .           do t338 = J1 + 1, xyz_qco2.DSC.U2, 4                           
     .              D2 = 1.D0/meanmolweight                                     
     .              D3 = 1.D0/meanmolweight                                     
     .              D4 = 1.D0/meanmolweight                                     
     .              D5 = 1.D0/meanmolweight                                     
     .  !CDIR       NODEP                                                       
     .              do t340 = 1, xyz_qco2.DSC.U1 + 1                            
     .                 xyz_qco2(t340-1,t338,t336+1) = ((vmrco2*co2molweight)*D2)
     .                 xyz_qco2(t340-1,t338+1,t336+1) = ((vmrco2*co2molweight)* 
     .       1            D3)                                                   
     .                 xyz_qco2(t340-1,t338+2,t336+1) = ((vmrco2*co2molweight)* 
     .       1            D4)                                                   
     .                 xyz_qco2(t340-1,t338+3,t336+1) = ((vmrco2*co2molweight)* 
     .       1            D5)                                                   
     .              end do                                                      
     .           end do                                                         
     .        endif                                                             
   505  
   506  
   507  
   508      ! O3 ʬۤ
   509      ! Setting of O3 distribution
   510      !
   511      call SetO3( &
   512        & xyz_Press, & ! (in )
   513        & xyz_QO3    & ! (out)
   514        & )
   515  
   516  
   517      xyz_DelCO2Mass    = xyz_DelAtmMass * xyz_QCO2
     .  !CDIR NODEP                                                             
     .        do t349 = 1, imax                                                 
     .           xyz_delco2mass(t349-1,t347+1,t345+1) = xyz_delatmmass(t349-1,  
     .       1      t347+1,t345+1)*xyz_qco2(t349-1,t347+1,t345+1)               
     .           xyz_delh2ovapmass(t349-1,t347+1,t345+1) = xyz_delatmmass(t349-1
     .       1      ,t347+1,t345+1)*xyz_qh2ovap(t349-1,t347+1,t345+1)           
     .           xyz_delh2oliqmass(t349-1,t347+1,t345+1) = xyz_delatmmass(t349-1
     .       1      ,t347+1,t345+1)*xyz_qh2oliq(t349-1,t347+1,t345+1)           
     .           xyz_delh2osolmass(t349-1,t347+1,t345+1) = xyz_delatmmass(t349-1
     .       1      ,t347+1,t345+1)*xyz_qh2osol(t349-1,t347+1,t345+1)           
     .           xyz_delo3mass(t349-1,t347+1,t345+1) = xyz_delatmmass(t349-1,   
     .       1      t347+1,t345+1)*xyz_qo3(xyz_qo3.DSC.L1+t349-1,t347+          
     .       2      xyz_qo3.DSC.L2,t345+xyz_qo3.DSC.L3)                         
     .        end do                                                            
   518      xyz_DelH2OVapMass = xyz_DelAtmMass * xyz_QH2OVap
   519      xyz_DelH2OLiqMass = xyz_DelAtmMass * xyz_QH2OLiq
   520      xyz_DelH2OSolMass = xyz_DelAtmMass * xyz_QH2OSol
   521      xyz_DelO3Mass     = xyz_DelAtmMass * xyz_QO3
   522  
   523  
   524  !!$    select case ( SWVer )
   525  !!$    case ( 0 )
   526  !!$      ! dcpam ϵ絤ûͥǥ Ver. 2
   527  !!$      ! dcpam short wave radiation model for the Earth's atmosphere Ver. 2
   528  !!$      !
   529  !!$      call RadiationDcpamESWV2Flux(                                 &
   530  !!$        & xy_SurfAlbedo,                                                  &
   531  !!$        & xyz_Press, xyr_Press, xyz_Temp, xyz_QH2OVap, xyz_QO3, xyz_Height,  &
   532  !!$        & xyr_RadSFlux                                                    &
   533  !!$        & )
   534  !!$    case ( 1 )
   535        ! ϵ絤ûͥǥ Ver. 2.1
   536        ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   537        !
   538        call RadEarthSWV21Flux(                                      &
   539          & xy_SurfAlbedo,                                           &
   540          & xyz_DelAtmMass,                                          &
   541          & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
   542          & xyz_DelO3Mass,                                           &
   543          & xyz_Press, xyz_Temp,                                     &
   544          & xyz_CloudCover,                                          &
   545          & xyr_RadSUwFlux, xyr_RadSDwFlux                           &
   546          & )
   547  !!$    case ( 2 )
   548  !!$      ! ϵ絤ûͥǥ Ver. 2.2
   549  !!$      ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   550  !!$      !
   551  !!$      call RadEarthSWV22Flux(                                      &
   552  !!$        & xy_SurfAlbedo,                                           &
   553  !!$        & xyz_DelAtmMass,                                          &
   554  !!$        & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
   555  !!$        & xyz_DelO3Mass,                                           &
   556  !!$        & xyz_Press, xyz_Temp,                                     &
   557  !!$        & xyz_CloudCover,                                          &
   558  !!$        & xyr_RadSUwFlux, xyr_RadSDwFlux                           &
   559  !!$        & )
   560  !!$    case default
   561  !!$      call MessageNotify( 'E', module_name, 'SW model version %d is not supported.', i = (/ LWVer /) )
   562  !!$    end select
   563  
   564  !!$    select case ( LWVer )
   565  !!$    case ( 0 )
   566  !!$      call RadiationDcpamELWV2Flux( &
   567  !!$        & xyz_Press, xyr_Press, xyz_Temp, xy_SurfTemp,                    & ! (in )
   568  !!$        & xyz_QH2OVap, xyz_QO3,                                           & ! (in )
   569  !!$        & xyr_RadLFlux, xyra_DelRadLFlux                                  & ! (out)
   570  !!$        & )
   571  !!$    case ( 1 )
   572  !!$      call RadiationDcpamELWV21Flux( &
   573  !!$        & xyz_Press, xyr_Press, xyz_Temp, xy_SurfTemp,                    & ! (in )
   574  !!$        & xyz_QH2OVap, xyz_QO3,                                           & ! (in )
   575  !!$        & xyr_RadLFlux, xyra_DelRadLFlux                                  & ! (out)
   576  !!$        & )
   577  !!$    case ( 2 )
   578  !!$      call RadiationDcpamELWV22Flux( &
   579  !!$        & xyz_Press, xyr_Press, xyz_Temp, xy_SurfTemp,                    & ! (in )
   580  !!$        & xyz_QH2OVap, xyz_QO3,                                           & ! (in )
   581  !!$        & xyr_RadLFlux, xyra_DelRadLFlux                                  & ! (out)
   582  !!$        & )
   583  !!$    case ( 3 )
   584        call RadEarthLWV23Flux(                                      &
   585          & xyz_DelCO2Mass,                                          &
   586          & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
   587          & xyz_DelO3Mass,                                           &
   588          & xyz_Press, xyz_Temp, xy_SurfTemp,                        & ! (in )
   589          & xyz_QCO2, xyz_QH2OVap,                                   & ! (in )
   590          & xyz_CloudCover,                                          & ! (in )
   591          & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
   592          & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
   593          & )
   594  !!$    case default
   595  !!$      call MessageNotify( 'E', module_name, 'LW model version %d is not supported.', i = (/ LWVer /) )
   596  !!$    end select
   597  
   598  
   599      ! Output variables
   600      !
   601      call HistoryAutoPut( TimeN, 'CloudCoverforRad', xyz_CloudCover )
   602  
   603  
   604    end subroutine RadEarthV2FluxCore
   605  
   606    !--------------------------------------------------------------------------------------
   607  
   608    subroutine RadEarthV2Init(    &
   609      & FlagSnow                  &
   610      & )
   611  
   612      ! ե
   613      ! File I/O support
   614      !
   615      use dc_iounit, only: FileOpen
   616  
   617      ! ҥȥǡ
   618      ! History data output
   619      !
   620      use gtool_historyauto, only: HistoryAutoAddVariable
   621  
   622      ! NAMELIST եϤ˴ؤ桼ƥƥ
   623      ! Utilities for NAMELIST file input
   624      !
   625      use namelist_util, only: namelist_filename, NmlutilMsg, NmlutilAryValid
   626  
   627      ! O3 ʬۤ
   628      ! Set O3 distribution
   629      !
   630      use set_o3, only : SetO3Init
   631  
   632      ! ϵ絤ûͥǥ Ver. 2.1
   633      ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   634      !
   635      use rad_Earth_SW_V2_1, only: RadEarthSWV21Init
   636  
   637  !!$    ! ϵ絤ûͥǥ Ver. 2.2
   638  !!$    ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   639  !!$    !
   640  !!$    use rad_Earth_SW_V2_2, only: RadEarthSWV22Init
   641  
   642      ! ϵ絤Ĺͥǥ Ver. 2.3
   643      ! long wave radiation model for the Earth's atmosphere Ver. 2.3
   644      !
   645      use rad_Earth_LW_V2_3, only : RadEarthLWV23Init
   646  
   647  
   648      ! ʸ ; Declaration statements
   649      !
   650      logical, intent(in) :: FlagSnow
   651  
   652  
   653      integer:: unit_nml        ! NAMELIST ե륪ץֹ.
   654                                ! Unit number for NAMELIST file open
   655      integer:: iostat_nml      ! NAMELIST ɤ߹߻ IOSTAT.
   656                                ! IOSTAT of NAMELIST read
   657  
   658      ! NAMELIST ѿ
   659      ! NAMELIST group name
   660      !
   661      namelist /rad_Earth_V2_nml/ &
   662        & VMRCO2
   663  
   664  !!$      & SWVer, LWVer
   665            !
   666            ! ǥեͤˤĤƤϽ³ "rad_Earth_V2#RadEarthV2Init"
   667            ! Υɤ򻲾ȤΤ.
   668            !
   669            ! Refer to source codes in the initialization procedure
   670            ! "rad_Earth_V2#RadEarthV2Init" for the default values.
   671            !
   672  
   673      if ( rad_Earth_V2_inited ) return
   674  
   675  
   676      ! Molecular weights of atmosphere, CO2, and H2O
   677      !
   678      MeanMolWeight = 28.0d-3
   679      CO2MolWeight  = 44.0d-3
   680      H2OMolWeight  = 18.0d-3
   681  
   682  
   683      ! ǥեͤ
   684      ! Default values settings
   685      !
   686      VMRCO2                = 382.0d-6
   687  
   688  
   689  !!$    SWVer = 1
   690  !!$    LWVer = 3
   691  
   692  
   693      ! NAMELIST ɤ߹
   694      ! NAMELIST is input
   695      !
   696      if ( trim(namelist_filename) /= '' ) then
   697        call FileOpen( unit_nml, &          ! (out)
   698          & namelist_filename, mode = 'r' ) ! (in)
   699  
   700        rewind( unit_nml )
   701        read( unit_nml,                     & ! (in)
   702          & nml = rad_Earth_V2_nml,         & ! (out)
   703          & iostat = iostat_nml )             ! (out)
   704        close( unit_nml )
   705  
   706        call NmlutilMsg( iostat_nml, module_name ) ! (in)
   707      end if
   708  
   709  
   710      ! Allocate a local variable for O3 distribution
   711      !
   712      allocate( xyz_QO3(0:imax-1, 1:jmax, 1:kmax) )
   713  
   714  
   715      ! Initialization of modules used in this module
   716      !
   717  
   718      ! O3 ʬۤ
   719      ! Set O3 distribution
   720      !
   721      call SetO3Init
   722  
   723      ! ϵ絤ûͥǥ Ver. 2.1
   724      ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   725      !
   726      call RadEarthSWV21Init
   727  
   728  !!$    ! ϵ絤ûͥǥ Ver. 2.2
   729  !!$    ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   730  !!$    !
   731  !!$    call RadEarthSWV22Init( &
   732  !!$      & FlagSnow            &
   733  !!$      & )
   734  
   735      ! ϵ絤Ĺͥǥ Ver. 2.3
   736      ! long wave radiation model for the Earth's atmosphere Ver. 2.3
   737      !
   738      call RadEarthLWV23Init( &
   739        & FlagSnow            &
   740        & )
   741  
   742  
   743      ! ҥȥǡϤΤΤؤѿϿ
   744      ! Register of variables for history data output
   745      !
   746      call HistoryAutoAddVariable( 'CloudCoverforRad', &
   747        & (/ 'lon ', 'lat ', 'sig ', 'time' /), &
   748        & 'cloud cover', '1' )
   749  
   750  
   751      !  ; Print
   752      !
   753      call MessageNotify( 'M', module_name, '----- Initialization Messages -----' )
   754      call MessageNotify( 'M', module_name, '  VMRCO2 = %f', &
   755        & d = (/ VMRCO2 /) )
   756  !!$    call MessageNotify( 'M', module_name, 'SWVer = %d', i = (/ SWVer /) )
   757  !!$    call MessageNotify( 'M', module_name, 'LWVer = %d', i = (/ LWVer /) )
   758      call MessageNotify( 'M', module_name, '-- version = %c', c1 = trim(version) )
   759  
   760  
   761      rad_Earth_V2_inited = .true.
   762  
   763    end subroutine RadEarthV2Init
   764  
   765    !--------------------------------------------------------------------------------------
   766  
   767  end module rad_Earth_V2
Linux  R2.6.5-7.282-sn2 FORTRAN90/SX         Rev.360        Mon Oct  1 21:50:41 2012
FILE NAME: rad_Earth_V2.f90
PROGRAM NAME: rad_earth_v2
FORMAT LIST

  LINE    LOOP     FORTRAN STATEMENT

     1:            != ϵ絤ͥǥ Ver. 2
     2:            !
     3:            != radiation model for the Earth's atmosphere Ver. 2
     4:            !
     5:            ! Authors::   Yoshiyuki O. Takahashi
     6:            ! Version::   $Id: rad_Earth_V2.f90,v 1.6 2012-09-08 15:25:05 yot Exp $
     7:            ! Tag Name::  $Name: dcpam5-20120921 $
     8:            ! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
     9:            ! License::   See COPYRIGHT[link:../../../COPYRIGHT]
    10:            !
    11:            module rad_Earth_V2
    12:              !
    13:              != ϵ絤ͥǥ Ver. 2
    14:              !
    15:              != radiation model for the Earth's atmosphere Ver. 1
    16:              !
    17:              ! <b>Note that Japanese and English are described in parallel.</b>
    18:              !
    19:              ! ϵ絤ͥǥ.
    20:              !
    21:              ! This is a radiation model for the Earth's atmospehre.
    22:              !
    23:              ! Radiation in the wavenumber range from    0 to  3000 cm-1 
    24:              ! is calculated in the routine of long wave radiation. 
    25:              ! Radiation in the wavenumber range from 1000 to 57143 cm-1 (0.175 to 10 micron) 
    26:              ! is calculated in the routine of shortwave radiation. 
    27:              !
    28:              !== References
    29:              !
    30:              !
    31:              !== Procedures List
    32:              !
    33:              ! RadEarthV2Flux :: ͥեåη׻
    34:              ! ------------          :: ------------
    35:              ! RadEarthV2Flux :: Calculate radiation flux
    36:              !
    37:              !== NAMELIST
    38:              !
    39:              ! NAMELIST#rad_Earth_V2_nml
    40:              !
    41:            
    42:              ! USE statements
    43:              !
    44:            
    45:              !
    46:              ! Kind type parameter
    47:              !
    48:              use dc_types, only: DP, &      ! Double precision.
    49:                &                 STRING, &  ! Strings.
    50:                &                 TOKEN      ! Keywords.
    51:            
    52:              ! å
    53:              ! Message output
    54:              !
    55:              use dc_message, only: MessageNotify
    56:            
    57:              !
    58:              ! Physical constants settings
    59:              !
    60:              use constants, only: Grav     ! $ g $ [m s-2].
    61:                                            !
    62:                                            ! Gravitational acceleration
    63:            
    64:              ! ʻ
    65:              ! Grid points settings
    66:              !
    67:              use gridset, only: imax, & ! ٳʻ.
    68:                                         ! Number of grid points in longitude
    69:                &                jmax, & ! ٳʻ.
    70:                                         ! Number of grid points in latitude
    71:                &                kmax    ! ľؿ.
    72:                                         ! Number of vertical level
    73:            
    74:              implicit none
    75:            
    76:              private
    77:            
    78:              real(DP), save :: VMRCO2            ! Volume mixing ratio of CO2
    79:              real(DP), save :: MeanMolWeight
    80:              real(DP), save :: H2OMolWeight
    81:              real(DP), save :: CO2MolWeight
    82:            
    83:              real(DP), allocatable, save :: xyz_QO3(:,:,:)
    84:                                          ! O3 ʬ (1)
    85:                                          ! O3 distribution (1)
    86:            
    87:            !!$  integer , save              :: SWVer
    88:            !!$  integer , save              :: LWVer
    89:            
    90:              ! ѿ
    91:              ! Public variables
    92:              !
    93:              logical, save :: rad_Earth_V2_inited = .false.
    94:                                          ! ե饰.
    95:                                          ! Initialization flag
    96:            
    97:              public :: RadEarthV2Flux
    98:            !!$  public :: RadEarthV2FluxforNHM
    99:              public :: RadEarthV2Init
   100:            
   101:              character(*), parameter:: module_name = 'rad_Earth_V2'
   102:                                          ! ⥸塼̾.
   103:                                          ! Module name
   104:              character(*), parameter:: version = &
   105:                & '$Name: dcpam5-20120921 $' // &
   106:                & '$Id: rad_Earth_V2.f90,v 1.6 2012-09-08 15:25:05 yot Exp $'
   107:                                          ! ⥸塼ΥС
   108:                                          ! Module version
   109:            
   110:              !--------------------------------------------------------------------------------------
   111:            
   112:            contains
   113:            
   114:              !--------------------------------------------------------------------------------------
   115:            
   116:              subroutine RadEarthV2Flux(                                                 &
   117:                & xy_SurfAlbedo,                                                         & ! (in)
   118:                & xyz_Press, xyr_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, & ! (in)
   119:                & xyz_CloudCover,                                                        & ! (in)
   120:                & xy_SurfTemp,                                                           & ! (in)
   121:                & xyr_RadSUwFlux, xyr_RadSDwFlux,                                        & ! (out)
   122:                & xyr_RadLUwFlux, xyr_RadLDwFlux,                                        & ! (out)
   123:                & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                                 & ! (out)
   124:                & )
   125:            
   126:            
   127:                ! USE statements
   128:                !
   129:            
   130:                real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   131:                real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   132:                real(DP), intent(in ) :: xyr_Press       (0:imax-1, 1:jmax, 0:kmax)
   133:                real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   134:                real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   135:                                          ! $ q $ .   . Mass mixing ratio of constituents (1)
   136:                real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   137:                real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
   138:                real(DP), intent(in ) :: xyz_CloudCover  (0:imax-1, 1:jmax, 1:kmax)
   139:                real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   140:                real(DP), intent(out) :: xyr_RadSUwFlux   (0:imax-1, 1:jmax, 0:kmax)
   141:                real(DP), intent(out) :: xyr_RadSDwFlux   (0:imax-1, 1:jmax, 0:kmax)
   142:                real(DP), intent(out) :: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   143:                real(DP), intent(out) :: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   144:                real(DP), intent(out) :: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   145:                real(DP), intent(out) :: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   146:            
   147:            !!$    real(DP):: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   148:            !!$    real(DP):: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   149:            !!$    real(DP):: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   150:            !!$    real(DP):: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   151:            
   152:            
   153:                ! Work variables
   154:                !
   155:                real(DP) :: xyz_DelAtmMass   (0:imax-1, 1:jmax, 1:kmax)
   156:            
   157:                integer  :: k
   158:            
   159:            
   160:                ! ǧ
   161:                ! Initialization check
   162:                !
   163:                if ( .not. rad_Earth_V2_inited ) then
   164:                  call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   165:                end if
   166:            
   167:            
   168: +------>       do k = 1, kmax
   169: |+V====          xyz_DelAtmMass(:,:,k) = ( xyr_Press(:,:,k-1) - xyr_Press(:,:,k ) ) / Grav
   170: +------        end do
   171:            
   172:                call RadEarthV2FluxCore(                                                        &
   173:                  & xy_SurfAlbedo,                                                              &
   174:                  & xyz_DelAtmMass, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   175:                  & xyz_CloudCover,                                                             &
   176:                  & xy_SurfTemp,                                                                &
   177:                  & xyr_RadSUwFlux, xyr_RadSDwFlux,                                             &
   178:                  & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
   179:                  & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
   180:                  & )
   181:            
   182:            
   183:              end subroutine RadEarthV2Flux
   184:            
   185:              !--------------------------------------------------------------------------------------
   186:            !!$
   187:            !!$  subroutine RadEarthV2FluxforNHM(                                          &
   188:            !!$    & r_Height, xy_SurfAlbedo,                                              &
   189:            !!$    & xyz_Dens, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   190:            !!$    & xy_SurfTemp,                                                          &
   191:            !!$    & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                          &
   192:            !!$    & )
   193:            !!$
   194:            !!$
   195:            !!$    ! USE statements
   196:            !!$    !
   197:            !!$
   198:            !!$    real(DP), intent(in ) :: r_Height                          (0:kmax)
   199:            !!$    real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   200:            !!$    real(DP), intent(in ) :: xyz_Dens        (0:imax-1, 1:jmax, 1:kmax)
   201:            !!$    real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   202:            !!$    real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   203:            !!$    real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   204:            !!$                              ! $ q $ .   . Mass mixing ratio of constituents (1)
   205:            !!$    real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   206:            !!$    real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
   207:            !!$    real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   208:            !!$    real(DP), intent(out) :: xyr_RadSFlux    (0:imax-1, 1:jmax, 0:kmax)
   209:            !!$    real(DP), intent(out) :: xyr_RadLFlux    (0:imax-1, 1:jmax, 0:kmax)
   210:            !!$    real(DP), intent(out) :: xyra_DelRadLFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   211:            !!$
   212:            !!$
   213:            !!$    ! Work variables
   214:            !!$    !
   215:            !!$    real(DP) :: xyz_DelAtmMass   (0:imax-1, 1:jmax, 1:kmax)
   216:            !!$
   217:            !!$    integer  :: k
   218:            !!$
   219:            !!$
   220:            !!$    ! ǧ
   221:            !!$    ! Initialization check
   222:            !!$    !
   223:            !!$    if ( .not. rad_Earth_V2_inited ) then
   224:            !!$      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   225:            !!$    end if
   226:            !!$
   227:            !!$
   228:            !!$    do k = 1, kmax
   229:            !!$      xyz_DelAtmMass(:,:,k) = xyz_Dens(:,:,k) * ( r_Height(k) - r_Height(k-1) )
   230:            !!$    end do
   231:            !!$
   232:            !!$    call RadEarthV2FluxCore(                                                        &
   233:            !!$      & xy_SurfAlbedo,                                                              &
   234:            !!$      & xyz_DelAtmMass, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   235:            !!$      & xy_SurfTemp,                                                                &
   236:            !!$      & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                                &
   237:            !!$      & )
   238:            !!$
   239:            !!$
   240:            !!$  end subroutine RadEarthV2FluxforNHM
   241:            !!$
   242:              !--------------------------------------------------------------------------------------
   243:            !!$
   244:            !!$  subroutine RadDcpamEV2FluxforNHM2DWrapper(                    &
   245:            !!$    & r_Height, x_SurfAlbedo,                                   &
   246:            !!$    & xz_Dens, xz_Press, xz_Temp, xz_QH2OVap, xz_QH2OLiq,       &
   247:            !!$    & x_SurfTemp,                                               &
   248:            !!$    & xr_RadSFlux, xr_RadLFlux, xra_DelRadLFlux                 &
   249:            !!$    & )
   250:            !!$
   251:            !!$
   252:            !!$    ! USE statements
   253:            !!$    !
   254:            !!$
   255:            !!$
   256:            !!$    real(DP), intent(in ) :: r_Height                 (0:kmax)
   257:            !!$    real(DP), intent(in ) :: x_SurfAlbedo   (0:imax-1)
   258:            !!$    real(DP), intent(in ) :: xz_Dens        (0:imax-1, 1:kmax)
   259:            !!$    real(DP), intent(in ) :: xz_Press       (0:imax-1, 1:kmax)
   260:            !!$    real(DP), intent(in ) :: xz_Temp        (0:imax-1, 1:kmax)
   261:            !!$    real(DP), intent(in ) :: xz_QH2OVap     (0:imax-1, 1:kmax)
   262:            !!$    real(DP), intent(in ) :: xz_QH2OLiq     (0:imax-1, 1:kmax)
   263:            !!$    real(DP), intent(in ) :: x_SurfTemp     (0:imax-1)
   264:            !!$    real(DP), intent(out) :: xr_RadSFlux    (0:imax-1, 0:kmax)
   265:            !!$    real(DP), intent(out) :: xr_RadLFlux    (0:imax-1, 0:kmax)
   266:            !!$    real(DP), intent(out) :: xra_DelRadLFlux(0:imax-1, 0:kmax, 0:1)
   267:            !!$
   268:            !!$
   269:            !!$    ! Work variables
   270:            !!$    !
   271:            !!$    real(DP) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   272:            !!$    real(DP) :: xyz_Dens        (0:imax-1, 1:jmax, 1:kmax)
   273:            !!$    real(DP) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   274:            !!$    real(DP) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   275:            !!$    real(DP) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   276:            !!$    real(DP) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   277:            !!$    real(DP) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   278:            !!$    real(DP) :: xyr_RadSFlux    (0:imax-1, 1:jmax, 0:kmax)
   279:            !!$    real(DP) :: xyr_RadLFlux    (0:imax-1, 1:jmax, 0:kmax)
   280:            !!$    real(DP) :: xyra_DelRadLFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   281:            !!$
   282:            !!$    integer  :: k
   283:            !!$
   284:            !!$
   285:            !!$    ! ǧ
   286:            !!$    ! Initialization check
   287:            !!$    !
   288:            !!$    if ( .not. rad_dcpam_E_V2_inited ) then
   289:            !!$      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   290:            !!$    end if
   291:            !!$
   292:            !!$
   293:            !!$    if ( jmax /= 1 ) then
   294:            !!$      call MessageNotify( 'E', module_name, 'jmax must be 1.' )
   295:            !!$    end if
   296:            !!$
   297:            !!$
   298:            !!$    xy_SurfAlbedo   (:, 1)       = x_SurfAlbedo   (:)
   299:            !!$    do k = 1, kmax
   300:            !!$      xyz_Dens        (:, 1, k)    = xz_Dens        (:, k)
   301:            !!$      xyz_Press       (:, 1, k)    = xz_Press       (:, k)
   302:            !!$      xyz_Temp        (:, 1, k)    = xz_Temp        (:, k)
   303:            !!$      xyz_QH2OVap     (:, 1, k)    = xz_QH2OVap     (:, k)
   304:            !!$      xyz_QH2OLiq     (:, 1, k)    = xz_QH2OLiq     (:, k)
   305:            !!$    end do
   306:            !!$    xy_SurfTemp     (:, 1)       = x_SurfTemp     (:)
   307:            !!$
   308:            !!$    call RadDcpamEV2FluxforNHM(                                       &
   309:            !!$      & r_Height, xy_SurfAlbedo,                                      &
   310:            !!$      & xyz_Dens, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq,      &
   311:            !!$      & xy_SurfTemp,                                                  &
   312:            !!$      & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                  &
   313:            !!$      & )
   314:            !!$
   315:            !!$    xr_RadSFlux    (:, :)    = xyr_RadSFlux    (:, 1, :)
   316:            !!$    xr_RadLFlux    (:, :)    = xyr_RadLFlux    (:, 1, :)
   317:            !!$    xra_DelRadLFlux(:, :, :) = xyra_DelRadLFlux(:, 1, :, :)
   318:            !!$
   319:            !!$
   320:            !!$  end subroutine RadDcpamEV2FluxforNHM2DWrapper
   321:            !!$
   322:              !--------------------------------------------------------------------------------------
   323:            !!$
   324:            !!$  subroutine RadDcpamEV2Flux2DWrapper(                         &
   325:            !!$    & x_SurfAlbedo,                                            &
   326:            !!$    & xz_Press, xr_Press, xz_Temp, xz_QH2OVap, xz_QH2OLiq,     &
   327:            !!$    & x_SurfTemp,                                              &
   328:            !!$    & xr_RadSFlux, xr_RadLFlux, xra_DelRadLFlux                &
   329:            !!$    & )
   330:            !!$
   331:            !!$
   332:            !!$    ! USE statements
   333:            !!$    !
   334:            !!$
   335:            !!$
   336:            !!$
   337:            !!$    real(DP), intent(in ) :: x_SurfAlbedo   (0:imax-1)
   338:            !!$    real(DP), intent(in ) :: xz_Press       (0:imax-1, 1:kmax)
   339:            !!$    real(DP), intent(in ) :: xr_Press       (0:imax-1, 0:kmax)
   340:            !!$    real(DP), intent(in ) :: xz_Temp        (0:imax-1, 1:kmax)
   341:            !!$    real(DP), intent(in ) :: xz_QH2OVap     (0:imax-1, 1:kmax)
   342:            !!$    real(DP), intent(in ) :: xz_QH2OLiq     (0:imax-1, 1:kmax)
   343:            !!$    real(DP), intent(in ) :: x_SurfTemp     (0:imax-1)
   344:            !!$    real(DP), intent(out) :: xr_RadSFlux    (0:imax-1, 0:kmax)
   345:            !!$    real(DP), intent(out) :: xr_RadLFlux    (0:imax-1, 0:kmax)
   346:            !!$    real(DP), intent(out) :: xra_DelRadLFlux(0:imax-1, 0:kmax, 0:1)
   347:            !!$
   348:            !!$
   349:            !!$    ! Work variables
   350:            !!$    !
   351:            !!$    real(DP) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   352:            !!$    real(DP) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   353:            !!$    real(DP) :: xyr_Press       (0:imax-1, 1:jmax, 0:kmax)
   354:            !!$    real(DP) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   355:            !!$    real(DP) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   356:            !!$    real(DP) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   357:            !!$    real(DP) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   358:            !!$    real(DP) :: xyr_RadSFlux    (0:imax-1, 1:jmax, 0:kmax)
   359:            !!$    real(DP) :: xyr_RadLFlux    (0:imax-1, 1:jmax, 0:kmax)
   360:            !!$    real(DP) :: xyra_DelRadLFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   361:            !!$
   362:            !!$
   363:            !!$    ! ǧ
   364:            !!$    ! Initialization check
   365:            !!$    !
   366:            !!$    if ( .not. rad_dcpam_E_V2_inited ) then
   367:            !!$      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   368:            !!$    end if
   369:            !!$
   370:            !!$
   371:            !!$    if ( jmax /= 1 ) then
   372:            !!$      call MessageNotify( 'E', module_name, 'jmax must be 1.' )
   373:            !!$    end if
   374:            !!$
   375:            !!$
   376:            !!$    xy_SurfAlbedo   (:, 1)       = x_SurfAlbedo   (:)
   377:            !!$    xyz_Press       (:, 1, :)    = xz_Press       (:, :)
   378:            !!$    xyr_Press       (:, 1, :)    = xr_Press       (:, :)
   379:            !!$    xyz_Temp        (:, 1, :)    = xz_Temp        (:, :)
   380:            !!$    xyz_QH2OVap     (:, 1, :)    = xz_QH2OVap     (:, :)
   381:            !!$    xyz_QH2OLiq     (:, 1, :)    = xz_QH2OLiq     (:, :)
   382:            !!$    xy_SurfTemp     (:, 1)       = x_SurfTemp     (:)
   383:            !!$
   384:            !!$    call RadDcpamEV2Flux(                                             &
   385:            !!$      & xy_SurfAlbedo,                                                &
   386:            !!$      & xyz_Press, xyr_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq,     &
   387:            !!$      & xy_SurfTemp,                                                  &
   388:            !!$      & xyr_RadSFlux, xyr_RadLFlux, xyra_DelRadLFlux                  &
   389:            !!$      & )
   390:            !!$
   391:            !!$    xr_RadSFlux    (:, :)    = xyr_RadSFlux    (:, 1, :)
   392:            !!$    xr_RadLFlux    (:, :)    = xyr_RadLFlux    (:, 1, :)
   393:            !!$    xra_DelRadLFlux(:, :, :) = xyra_DelRadLFlux(:, 1, :, :)
   394:            !!$
   395:            !!$
   396:            !!$  end subroutine RadDcpamEV2Flux2DWrapper
   397:            !!$
   398:              !--------------------------------------------------------------------------------------
   399:            
   400:              subroutine RadEarthV2FluxCore(                                                  &
   401:                & xy_SurfAlbedo,                                                              &
   402:                & xyz_DelAtmMass, xyz_Press, xyz_Temp, xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
   403:                & xyz_CloudCover,                                                             &
   404:                & xy_SurfTemp,                                                                &
   405:                & xyr_RadSUwFlux, xyr_RadSDwFlux,                                             &
   406:                & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
   407:                & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
   408:                & )
   409:            
   410:            
   411:                ! USE statements
   412:                !
   413:            
   414:                ! ҥȥǡ
   415:                ! History data output
   416:                !
   417:                use gtool_historyauto, only: HistoryAutoPut
   418:            
   419:                ! 
   420:                ! Time control
   421:                !
   422:                use timeset, only: &
   423:                  & TimeN                 ! ƥå $ t $ λ.
   424:                                          ! Time of step $ t $.
   425:            
   426:                ! O3 ʬۤ
   427:                ! Set O3 distribution
   428:                !
   429:                use set_o3, only: SetO3
   430:            
   431:            !!$    ! dcpam ϵ絤ûͥǥ Ver. 2
   432:            !!$    ! dcpam short wave radiation model for the Earth's atmosphere Ver. 2
   433:            !!$    !
   434:            !!$    use radiation_dcpam_E_SW_V2, only: RadiationDcpamESWV2Flux
   435:            
   436:                ! ϵ絤ûͥǥ Ver. 2.1
   437:                ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   438:                !
   439:                use rad_Earth_SW_V2_1, only: RadEarthSWV21Flux
   440:            !!$
   441:            !!$    ! ϵ絤ûͥǥ Ver. 2.2
   442:            !!$    ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   443:            !!$    !
   444:            !!$    use rad_Earth_SW_V2_2, only: RadEarthSWV22Flux
   445:            
   446:            !!$    ! dcpam ϵ絤Ĺͥǥ Ver. 2
   447:            !!$    ! dcpam long wave radiation model for the Earth's atmosphere Ver. 2
   448:            !!$    !
   449:            !!$    use radiation_dcpam_E_LW_V2, only : RadiationDcpamELWV2Flux
   450:            !!$
   451:            !!$    ! dcpam ϵ絤Ĺͥǥ Ver. 2.1
   452:            !!$    ! dcpam long wave radiation model for the Earth's atmosphere Ver. 2.1
   453:            !!$    !
   454:            !!$    use radiation_dcpam_E_LW_V2_1, only : RadiationDcpamELWV21Flux
   455:            !!$
   456:            !!$    ! dcpam ϵ絤Ĺͥǥ Ver. 2.2
   457:            !!$    ! dcpam long wave radiation model for the Earth's atmosphere Ver. 2.2
   458:            !!$    !
   459:            !!$    use radiation_dcpam_E_LW_V2_2, only : RadiationDcpamELWV22Flux
   460:            
   461:                ! ϵ絤Ĺͥǥ Ver. 2.3
   462:                ! long wave radiation model for the Earth's atmosphere Ver. 2.3
   463:                !
   464:                use rad_Earth_LW_V2_3, only : RadEarthLWV23Flux
   465:            
   466:                real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
   467:                real(DP), intent(in ) :: xyz_DelAtmMass  (0:imax-1, 1:jmax, 1:kmax)
   468:                real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
   469:                real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
   470:                real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
   471:                                          ! $ q $ .   . Mass mixing ratio of constituents (1)
   472:                real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
   473:                real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
   474:                real(DP), intent(in ) :: xyz_CloudCover  (0:imax-1, 1:jmax, 1:kmax)
   475:                real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
   476:                real(DP), intent(out) :: xyr_RadSUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   477:                real(DP), intent(out) :: xyr_RadSDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   478:                real(DP), intent(out) :: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
   479:                real(DP), intent(out) :: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
   480:                real(DP), intent(out) :: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   481:                real(DP), intent(out) :: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
   482:            
   483:            
   484:                ! Work variables
   485:                !
   486:                real(DP) :: xyz_QCO2         (0:imax-1, 1:jmax, 1:kmax)
   487:            
   488:                real(DP) :: xyz_DelCO2Mass   (0:imax-1, 1:jmax, 1:kmax)
   489:                real(DP) :: xyz_DelH2OVapMass(0:imax-1, 1:jmax, 1:kmax)
   490:                real(DP) :: xyz_DelH2OLiqMass(0:imax-1, 1:jmax, 1:kmax)
   491:                real(DP) :: xyz_DelH2OSolMass(0:imax-1, 1:jmax, 1:kmax)
   492:                real(DP) :: xyz_DelO3Mass    (0:imax-1, 1:jmax, 1:kmax)
   493:            
   494:            
   495:            
   496:                ! ǧ
   497:                ! Initialization check
   498:                !
   499:                if ( .not. rad_Earth_V2_inited ) then
   500:                  call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   501:                end if
   502:            
   503:            
   504: ++V====        xyz_QCO2 = VMRCO2 * CO2MolWeight / MeanMolWeight
   505:            
   506:            
   507:            
   508:                ! O3 ʬۤ
   509:                ! Setting of O3 distribution
   510:                !
   511:                call SetO3( &
   512:                  & xyz_Press, & ! (in )
   513:                  & xyz_QO3    & ! (out)
   514:                  & )
   515:            
   516:            
   517: **V---->       xyz_DelCO2Mass    = xyz_DelAtmMass * xyz_QCO2
   518: |||            xyz_DelH2OVapMass = xyz_DelAtmMass * xyz_QH2OVap
   519: |||            xyz_DelH2OLiqMass = xyz_DelAtmMass * xyz_QH2OLiq
   520: |||            xyz_DelH2OSolMass = xyz_DelAtmMass * xyz_QH2OSol
   521: **V----        xyz_DelO3Mass     = xyz_DelAtmMass * xyz_QO3
   522:            
   523:            
   524:            !!$    select case ( SWVer )
   525:            !!$    case ( 0 )
   526:            !!$      ! dcpam ϵ絤ûͥǥ Ver. 2
   527:            !!$      ! dcpam short wave radiation model for the Earth's atmosphere Ver. 2
   528:            !!$      !
   529:            !!$      call RadiationDcpamESWV2Flux(                                 &
   530:            !!$        & xy_SurfAlbedo,                                                  &
   531:            !!$        & xyz_Press, xyr_Press, xyz_Temp, xyz_QH2OVap, xyz_QO3, xyz_Height,  &
   532:            !!$        & xyr_RadSFlux                                                    &
   533:            !!$        & )
   534:            !!$    case ( 1 )
   535:                  ! ϵ絤ûͥǥ Ver. 2.1
   536:                  ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   537:                  !
   538:                  call RadEarthSWV21Flux(                                      &
   539:                    & xy_SurfAlbedo,                                           &
   540:                    & xyz_DelAtmMass,                                          &
   541:                    & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
   542:                    & xyz_DelO3Mass,                                           &
   543:                    & xyz_Press, xyz_Temp,                                     &
   544:                    & xyz_CloudCover,                                          &
   545:                    & xyr_RadSUwFlux, xyr_RadSDwFlux                           &
   546:                    & )
   547:            !!$    case ( 2 )
   548:            !!$      ! ϵ絤ûͥǥ Ver. 2.2
   549:            !!$      ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   550:            !!$      !
   551:            !!$      call RadEarthSWV22Flux(                                      &
   552:            !!$        & xy_SurfAlbedo,                                           &
   553:            !!$        & xyz_DelAtmMass,                                          &
   554:            !!$        & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
   555:            !!$        & xyz_DelO3Mass,                                           &
   556:            !!$        & xyz_Press, xyz_Temp,                                     &
   557:            !!$        & xyz_CloudCover,                                          &
   558:            !!$        & xyr_RadSUwFlux, xyr_RadSDwFlux                           &
   559:            !!$        & )
   560:            !!$    case default
   561:            !!$      call MessageNotify( 'E', module_name, 'SW model version %d is not supported.', i = (/ LWVer /) )
   562:            !!$    end select
   563:            
   564:            !!$    select case ( LWVer )
   565:            !!$    case ( 0 )
   566:            !!$      call RadiationDcpamELWV2Flux( &
   567:            !!$        & xyz_Press, xyr_Press, xyz_Temp, xy_SurfTemp,                    & ! (in )
   568:            !!$        & xyz_QH2OVap, xyz_QO3,                                           & ! (in )
   569:            !!$        & xyr_RadLFlux, xyra_DelRadLFlux                                  & ! (out)
   570:            !!$        & )
   571:            !!$    case ( 1 )
   572:            !!$      call RadiationDcpamELWV21Flux( &
   573:            !!$        & xyz_Press, xyr_Press, xyz_Temp, xy_SurfTemp,                    & ! (in )
   574:            !!$        & xyz_QH2OVap, xyz_QO3,                                           & ! (in )
   575:            !!$        & xyr_RadLFlux, xyra_DelRadLFlux                                  & ! (out)
   576:            !!$        & )
   577:            !!$    case ( 2 )
   578:            !!$      call RadiationDcpamELWV22Flux( &
   579:            !!$        & xyz_Press, xyr_Press, xyz_Temp, xy_SurfTemp,                    & ! (in )
   580:            !!$        & xyz_QH2OVap, xyz_QO3,                                           & ! (in )
   581:            !!$        & xyr_RadLFlux, xyra_DelRadLFlux                                  & ! (out)
   582:            !!$        & )
   583:            !!$    case ( 3 )
   584:                  call RadEarthLWV23Flux(                                      &
   585:                    & xyz_DelCO2Mass,                                          &
   586:                    & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
   587:                    & xyz_DelO3Mass,                                           &
   588:                    & xyz_Press, xyz_Temp, xy_SurfTemp,                        & ! (in )
   589:                    & xyz_QCO2, xyz_QH2OVap,                                   & ! (in )
   590:                    & xyz_CloudCover,                                          & ! (in )
   591:                    & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
   592:                    & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
   593:                    & )
   594:            !!$    case default
   595:            !!$      call MessageNotify( 'E', module_name, 'LW model version %d is not supported.', i = (/ LWVer /) )
   596:            !!$    end select
   597:            
   598:            
   599:                ! Output variables
   600:                !
   601:                call HistoryAutoPut( TimeN, 'CloudCoverforRad', xyz_CloudCover )
   602:            
   603:            
   604:              end subroutine RadEarthV2FluxCore
   605:            
   606:              !--------------------------------------------------------------------------------------
   607:            
   608:              subroutine RadEarthV2Init(    &
   609:                & FlagSnow                  &
   610:                & )
   611:            
   612:                ! ե
   613:                ! File I/O support
   614:                !
   615:                use dc_iounit, only: FileOpen
   616:            
   617:                ! ҥȥǡ
   618:                ! History data output
   619:                !
   620:                use gtool_historyauto, only: HistoryAutoAddVariable
   621:            
   622:                ! NAMELIST եϤ˴ؤ桼ƥƥ
   623:                ! Utilities for NAMELIST file input
   624:                !
   625:                use namelist_util, only: namelist_filename, NmlutilMsg, NmlutilAryValid
   626:            
   627:                ! O3 ʬۤ
   628:                ! Set O3 distribution
   629:                !
   630:                use set_o3, only : SetO3Init
   631:            
   632:                ! ϵ絤ûͥǥ Ver. 2.1
   633:                ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   634:                !
   635:                use rad_Earth_SW_V2_1, only: RadEarthSWV21Init
   636:            
   637:            !!$    ! ϵ絤ûͥǥ Ver. 2.2
   638:            !!$    ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   639:            !!$    !
   640:            !!$    use rad_Earth_SW_V2_2, only: RadEarthSWV22Init
   641:            
   642:                ! ϵ絤Ĺͥǥ Ver. 2.3
   643:                ! long wave radiation model for the Earth's atmosphere Ver. 2.3
   644:                !
   645:                use rad_Earth_LW_V2_3, only : RadEarthLWV23Init
   646:            
   647:            
   648:                ! ʸ ; Declaration statements
   649:                !
   650:                logical, intent(in) :: FlagSnow
   651:            
   652:            
   653:                integer:: unit_nml        ! NAMELIST ե륪ץֹ.
   654:                                          ! Unit number for NAMELIST file open
   655:                integer:: iostat_nml      ! NAMELIST ɤ߹߻ IOSTAT.
   656:                                          ! IOSTAT of NAMELIST read
   657:            
   658:                ! NAMELIST ѿ
   659:                ! NAMELIST group name
   660:                !
   661:                namelist /rad_Earth_V2_nml/ &
   662:                  & VMRCO2
   663:            
   664:            !!$      & SWVer, LWVer
   665:                      !
   666:                      ! ǥեͤˤĤƤϽ³ "rad_Earth_V2#RadEarthV2Init"
   667:                      ! Υɤ򻲾ȤΤ.
   668:                      !
   669:                      ! Refer to source codes in the initialization procedure
   670:                      ! "rad_Earth_V2#RadEarthV2Init" for the default values.
   671:                      !
   672:            
   673:                if ( rad_Earth_V2_inited ) return
   674:            
   675:            
   676:                ! Molecular weights of atmosphere, CO2, and H2O
   677:                !
   678:                MeanMolWeight = 28.0d-3
   679:                CO2MolWeight  = 44.0d-3
   680:                H2OMolWeight  = 18.0d-3
   681:            
   682:            
   683:                ! ǥեͤ
   684:                ! Default values settings
   685:                !
   686:                VMRCO2                = 382.0d-6
   687:            
   688:            
   689:            !!$    SWVer = 1
   690:            !!$    LWVer = 3
   691:            
   692:            
   693:                ! NAMELIST ɤ߹
   694:                ! NAMELIST is input
   695:                !
   696:                if ( trim(namelist_filename) /= '' ) then
   697:                  call FileOpen( unit_nml, &          ! (out)
   698:                    & namelist_filename, mode = 'r' ) ! (in)
   699:            
   700:                  rewind( unit_nml )
   701:                  read( unit_nml,                     & ! (in)
   702:                    & nml = rad_Earth_V2_nml,         & ! (out)
   703:                    & iostat = iostat_nml )             ! (out)
   704:                  close( unit_nml )
   705:            
   706:                  call NmlutilMsg( iostat_nml, module_name ) ! (in)
   707:                end if
   708:            
   709:            
   710:                ! Allocate a local variable for O3 distribution
   711:                !
   712:                allocate( xyz_QO3(0:imax-1, 1:jmax, 1:kmax) )
   713:            
   714:            
   715:                ! Initialization of modules used in this module
   716:                !
   717:            
   718:                ! O3 ʬۤ
   719:                ! Set O3 distribution
   720:                !
   721:                call SetO3Init
   722:            
   723:                ! ϵ絤ûͥǥ Ver. 2.1
   724:                ! short wave radiation model for the Earth's atmosphere Ver. 2.1
   725:                !
   726:                call RadEarthSWV21Init
   727:            
   728:            !!$    ! ϵ絤ûͥǥ Ver. 2.2
   729:            !!$    ! short wave radiation model for the Earth's atmosphere Ver. 2.2
   730:            !!$    !
   731:            !!$    call RadEarthSWV22Init( &
   732:            !!$      & FlagSnow            &
   733:            !!$      & )
   734:            
   735:                ! ϵ絤Ĺͥǥ Ver. 2.3
   736:                ! long wave radiation model for the Earth's atmosphere Ver. 2.3
   737:                !
   738:                call RadEarthLWV23Init( &
   739:                  & FlagSnow            &
   740:                  & )
   741:            
   742:            
   743:                ! ҥȥǡϤΤΤؤѿϿ
   744:                ! Register of variables for history data output
   745:                !
   746:                call HistoryAutoAddVariable( 'CloudCoverforRad', &
   747:                  & (/ 'lon ', 'lat ', 'sig ', 'time' /), &
   748:                  & 'cloud cover', '1' )
   749:            
   750:            
   751:                !  ; Print
   752:                !
   753:                call MessageNotify( 'M', module_name, '----- Initialization Messages -----' )
   754:                call MessageNotify( 'M', module_name, '  VMRCO2 = %f', &
   755:                  & d = (/ VMRCO2 /) )
   756:            !!$    call MessageNotify( 'M', module_name, 'SWVer = %d', i = (/ SWVer /) )
   757:            !!$    call MessageNotify( 'M', module_name, 'LWVer = %d', i = (/ LWVer /) )
   758:                call MessageNotify( 'M', module_name, '-- version = %c', c1 = trim(version) )
   759:            
   760:            
   761:                rad_Earth_V2_inited = .true.
   762:            
   763:              end subroutine RadEarthV2Init
   764:            
   765:              !--------------------------------------------------------------------------------------
   766:            
   767:            end module rad_Earth_V2
