!= Υȴ夲
!
!= Mars Dust Lifting!

! Authors::   OGIHARA Hirotaka
! Version::   $Id: Mars_dust_lifting.f90,v 0.1 2012-10-01 16:18:31 yot Exp $ 
! Tag Name::  $Name: dcpam5-20120921$
! Copyright:: Copyright (C) GFD Dennou Club, 2008-2009. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]
!

module mars_dust_lifting
  !
  != Υȴ夲
  !
  != Surface flux
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! Kahre et al (2006) ˴ŤƲΥȴ夲Ψ׻. 
  !
  ! 
  !
  !== References
  !
  ! Melinda A. Kahre, James R. Murphy, and Robert M. Haberle
  ! Modeling the Martian dust cycle and surface dust
  ! reservoirs with the NASA Ames general circulation
  ! model, Journal of Geophysical Research, vol 111, E06008, 2006 
  !
  !== Procedures List
  !
  ! MarsDustLifting   :: Υȴ夲Ψη׻
  ! ------------      :: ------------
  !
  !== NAMELIST
  !
  ! NAMELIST#mars_dust_lifting_nml
  !

  ! ⥸塼 ; USE statements
  !

  ! ʻ
  ! Grid points settings
  !
  use gridset, only: imax, & ! ٳʻ. 
                             ! Number of grid points in longitude
    &                jmax, & ! ٳʻ. 
                             ! Number of grid points in latitude
    &                kmax    ! ľؿ. 
                             ! Number of vertical level


  ! ̷ѥ᥿
  ! Kind type parameter
  !
  use dc_types, only: DP, &      ! ټ¿. Double precision. 
    &                 STRING     ! ʸ.       Strings. 

  ! å
  ! Message output
  !
  use dc_message, only: MessageNotify

  ! ʸ ; Declaration statements
  !
  implicit none
  private

  ! ³
  ! Public procedure
  !
  public :: MarsDustLifting
  public :: MarsDustLiftingOutput
  public :: MarsDustLiftingInit

  ! ѿ
  ! Public variables
  !

  ! ѿ
  ! Private variables
  !
  logical, save :: mars_dust_lifting_inited = .false.
                              ! ե饰
                              ! Initialization flag


  real(DP), save:: StressThreshold
                            ! ϥȤ򴬤夲 (N m-2)
                            ! Wind stress threshold (N m-2)

  real(DP), save:: KMHEfficiencyFactor
                            ! Ψ(m-1)
                            ! Efficiency factor (m-1)
  real(DP), save:: NewmanEfficiencyFactor
                            ! Ψ(m-1)
                            ! Efficiency factor (m-1)

  character(*), parameter:: module_name = 'mars_dust_lifting'
                              ! ⥸塼̾. 
                              ! Module name
  character(*), parameter:: version = &
    & '$Name: dcpam5-20120921 $' // &
    & '$Id: mars_dust_lifting.f90,v 0.1 2012-10-01 00:26:41 yot Exp $'
                              ! ⥸塼ΥС
                              ! Module version


contains
  
  !-------------------------------------------------------------------------------------

  subroutine MarsDustLifting( &
    & xyr_MomFluxX, xyr_MomFluxY, xyr_Press, xyr_VirTemp, & ! (in)
    & xy_KMHMarsDustLifting, xy_NewmanMarsDustLifting          & ! (out)
    & )
  !
  ! Х륯ˡǵ᤿, ǮեåȤäƲΥȴ夲Ψ
  ! ׻ޤ.

    ! ⥸塼 ; USE statements
    !

    ! ɸǡ
    ! Axes data settings
    !
    use axesset, only: &
      & y_Lat                 ! $ \varphi $ [rad.] . . Latitude

    ! 
    ! Time control
    !
    use timeset, only: &
      & TimeN, &              ! ƥå $ t $ λ. Time of step $ t $. 
      & TimesetClockStart, TimesetClockStop

    ! ǥХåѥ桼ƥƥ
    ! Utilities for debug
    !
    use dc_trace, only: DbgMessage, BeginSub, EndSub

    ! ʪ
    ! Physical constants settings
    !
    use constants, only: &
      & Grav, &               ! $ g $ [m s-2]. 
                              ! ϲ®. 
                              ! Gravitational acceleration
      & GasRDry, &
                              ! $ R $ [J kg-1 K-1]. 
                              ! 絤ε. 
                              ! Gas constant of air
      & CpDry
                              ! $ C_p $ [J kg-1 K-1]. 
                              ! 絤갵Ǯ. 
                              ! Specific heat of air at constant pressure

    ! ʸ ; Declaration statements
    !
    implicit none
    real(DP), intent(in):: xyr_MomFluxX (0:imax-1, 1:jmax, 0:kmax)
                              ! ư̥եå. 
                              ! Eastward momentum flux
    real(DP), intent(in):: xyr_MomFluxY (0:imax-1, 1:jmax, 0:kmax)
                              ! ư̥եå. 
                              ! Northward momentum flux
    real(DP), intent(in):: xyr_VirTemp (0:imax-1, 1:jmax, 0:kmax)
                              ! $ T_v $ .  (Ⱦ٥). 
                              ! Virtual temperature (half level)
    real(DP), intent(in):: xyr_Press (0:imax-1, 1:jmax, 0:kmax)
                              ! $ p_s $ . ɽ̵ (Ⱦ٥). 
                              ! Surface pressure (half level)
    real(DP) :: xy_MomFluxSurf (0:imax-1, 1:jmax)
                              ! ɽ̱ư̥եå礭.(N m-2) 
                              ! momentum flux power
    real(DP) :: xy_SurfRho (0:imax-1, 1:jmax)
                              ! ɽ̩
 
    real(DP), intent(out):: xy_KMHMarsDustLifting(0:imax-1, 1:jmax)
                              !Υȴ夲Ψ
    real(DP), intent(out):: xy_NewmanMarsDustLifting(0:imax-1, 1:jmax)
                              !Υȴ夲Ψ
    
    ! ѿ
    ! Work variables
    !

    integer:: i               ! ˲ DO 롼Ѻѿ
                              ! Work variables for DO loop in longitude
    integer:: j               ! ˲ DO 롼Ѻѿ
                              ! Work variables for DO loop in latitude
!    integer:: k               ! ˲ DO 롼Ѻѿ
                              ! Work variables for DO loop in dimension of constituents

    ! ¹ʸ ; Executable statement
    ! ǧ
    ! Initialization check
    !
!    if ( .not. mars_dust_lifting_inited ) call MarsDustLiftingInit
    if ( .not. mars_dust_lifting_inited ) then
      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
    end if
  
!   ɽ̱Ϥ礭η׻

    xy_MomFluxSurf(:,:) = sqrt( xyr_MomFluxX(:,:,0)**2 + xyr_MomFluxY(:,:,0)**2 )

!write(10,*) "xyr_MomFluxX", xyr_MomFLuxX(0,1,0)
!write(10,*) "xyr_MomFluxY", xyr_MomFLuxY(0,1,0)
!write(10,*) "xy_MomFluxSurf=", xy_MomFluxSurf(0,1)
!write(10,*) "StressThreshold", StressThreshold
    ! ȴ夲(KMH )
!    do i = 0, imax-1
!      do j = 1, jmax
!        if(xy_MomFluxSurf(i,j) < StressThreshold) then
!          xy_MarsDustLifting(i,j) = 0.0_DP
!        else
!          xy_MarsDustLifting(i,j) =                           & 
!            & EfficiencyFactor *(( 10.0_DP)**6)                   &
!            & * ( 2.3_DP* 0.001_DP ) * ((xy_MomFluxSurf(i,j))**2) &
!            & * (((xy_MomFluxSurf(i,j)) - StressThreshold)       &
!            & / StressThreshold)
!        end if
!      end do
!    end do

    do i = 0, imax-1
      do j = 1, jmax
        if(xy_MomFluxSurf(i,j) < StressThreshold) then
          xy_KMHMarsDustLifting(i,j) = 0.0_DP
        else
          xy_KMHMarsDustLifting(i,j) =                                                      & 
            & ((xy_MomFluxSurf(i,j))**2) * (((xy_MomFluxSurf(i,j)) - StressThreshold)    &
            & / StressThreshold) ! KMH αϤι


          xy_KMHMarsDustLifting(i,j) = (2.3_DP * 0.001_DP) * xy_KMHMarsDustLifting(i,j) ! ϵ¬᤿ 2.3 x 10^-3 ݤ

!          xy_MarsDustLifting(i,j) = ((10.0_DP)**6) * xy_MarsDustLifting(i,j)      ! ñ̤ kg m-1 s-1 ˤ

          xy_KMHMarsDustLifting(i,j) = KMHEfficiencyFactor * xy_KMHMarsDustLifting(i,j)    ! ΨҤݤ
           
        end if
      end do
    end do


!    do i = 0, imax-1
!      do j = 1, jmax
!        if((xy_MomFluxSurf(i,j)*1000.0_DP) < StressThreshold) then
!          xy_MarsDustLifting(i,j) = 0.0_DP
!        else
!          xy_MarsDustLifting(i,j) =                           & 
!            & EfficiencyFactor                                 &
!            & * ( 2.3_DP* 0.01_DP ) * ((xy_MomFluxSurf(i,j)* 1000.0_DP)**2) &
!            & * (((xy_MomFluxSurfX(i,j)*1000.0_DP) - StressThreshold)       &
!            & / StressThreshold)
!        end if
!      end do
!    end do


    !ɽ̩٤η׻
    do i = 0, imax-1
      do j = 1, jmax
        xy_SurfRho(i,j) = xyr_Press(i,j,0) /(GasRDry * xyr_VirTemp(i,j,0))
      end do
    end do
    ! Ϥˤȴ夲(Newman )

    do i = 0, imax-1
      do j = 1, jmax
        if(xy_MomFluxSurf(i,j) < StressThreshold) then
          xy_NewmanMarsDustLifting(i,j) = 0.0_DP
        else
          xy_NewmanMarsDustLifting(i,j) =                                 & 
            & ((xy_MomFluxSurf(i,j))**1.5)/(Grav*sqrt(xy_SurfRho(i,j))) * &
            & * ( 1.0_DP - sqrt(StressThreshold/xy_MomFluxSurf(i,j)))     &
            & * ( 1.0_DP + sqrt(StressThreshold/xy_MomFluxSurf(i,j)))**2 ! Newman αϤι

          xy_NewmanMarsDustLifting(i,j) = 2.61_DP * xy_NewmanMarsDustLifting(i,j) ! ƶ¸᤿ 2.61ݤ

          xy_NewmanMarsDustLifting(i,j) = NewmanEfficiencyFactor * xy_NewmanMarsDustLifting(i,j)    ! ΨҤݤ
           
        end if
      end do
    end do
  
  end subroutine MarsDustLifting
  !--------------------------------------------------------------------------------------

  subroutine MarsDustLiftingOutput(      &
    & xy_KMHMarsDustLifting, xy_NewmanMarsDustLifting                 & ! (in)
    & )
    !
    ! եå (xyr_KMHMarsDustLifting, xyr_NewmanMarsDustLifting). 
    ! ˤĤƽϤԤ. 
    !
    !

    ! ⥸塼 ; USE statements
    !

    ! 
    ! Time control
    !
    use timeset, only: &
      & TimeN, &            ! ƥå $ t $ λ. Time of step $ t $.
      & TimesetClockStart, TimesetClockStop

    ! ҥȥǡ
    ! History data output
    !
    use gtool_historyauto, only: HistoryAutoPut

    ! ʸ ; Declaration statements
    !
    implicit none
    real(DP), intent(in):: xy_KMHMarsDustLifting (0:imax-1, 1:jmax)
                              ! ȴ夲Ψ. 
                              !  momentum flux
    real(DP), intent(in):: xy_NewmanMarsDustLifting (0:imax-1, 1:jmax)
                              ! ȴ夲Ψ. 
                              !  momentum flux

    ! ϤΤκѿ
    real(DP):: xy_KMHDustLifting (0:imax-1, 1:jmax)
    real(DP):: xy_NewmanDustLifting (0:imax-1, 1:jmax)
    
    ! ѿ
    ! Work variables
    !
!    integer:: i               ! ˲ DO 롼Ѻѿ
                              ! Work variables for DO loop in longitude
!    integer:: j               ! ˲ DO 롼Ѻѿ
                              ! Work variables for DO loop in latitude


    ! ¹ʸ ; Executable statement
    !

    ! ׻ַ¬
    ! Start measurement of computation time
    !
    call TimesetClockStart( module_name )


    ! 
    ! Initialization
    !
    if ( .not. mars_dust_lifting_inited ) then
      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
    end if
!    if ( .not. mars_dust_lifting_inited ) call MarsDustLiftingInit
    xy_KMHDustLifting(:,:) = xy_KMHMarsDustLifting(:,:)
    xy_NewmanDustLifting(:,:) = xy_NewmanMarsDustLifting(:,:)
    
!do i=0, imax-1
!  do j=1, jmax
!    write(10,*) "DustLifting=", xy_DustLifting(i,j)
!  end do
!end do

    ! ҥȥǡ
    ! History data output
    !
    call HistoryAutoPut( TimeN, 'KMHF_d'           , xy_KMHDustLifting(:,:))
    call HistoryAutoPut( TimeN, 'NewmanF_d'           , xy_NewmanDustLifting(:,:))
!    call HistoryAutoPut( TimeN, 'F_dX' , xy_MarsDustLiftingX )
!    call HistoryAutoPut( TimeN, 'F_dY' , xy_MarsDustLiftingY )

    ! ׻ַ¬
    ! Pause measurement of computation time
    !
    call TimesetClockStop( module_name )

  end subroutine MarsDustLiftingOutput

  !--------------------------------------------------------------------------------------

  subroutine MarsDustLiftingInit
    !
    ! Mars_dust_lifting ⥸塼νԤޤ. 
    ! NAMELIST#surface_flux_bulk_nml ɤ߹ߤϤμ³ǹԤޤ. 
    !
    ! "surface_flux_bulk" module is initialized. 
    ! "NAMELIST#surface_flux_bulk_nml" is loaded in this procedure. 
    !

    ! ⥸塼 ; USE statements
    !

    ! NAMELIST եϤ˴ؤ桼ƥƥ
    ! Utilities for NAMELIST file input
    !
    use namelist_util, only: namelist_filename, NmlutilMsg, NmlutilAryValid

    ! ե
    ! File I/O support
    !
    use dc_iounit, only: FileOpen

    ! ̷ѥ᥿
    ! Kind type parameter
    !
    use dc_types, only: STDOUT ! ɸϤֹ. Unit number of standard output

    ! ʸ
    ! Character handling
    !
    use dc_string, only: StoA

    ! ҥȥǡ
    ! History data output
    !
    use gtool_historyauto, only: HistoryAutoAddVariable

    ! ʸ ; Declaration statements
    !
    implicit none

    integer:: unit_nml        ! NAMELIST ե륪ץֹ. 
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST ɤ߹߻ IOSTAT. 
                              ! IOSTAT of NAMELIST read

    ! NAMELIST ѿ
    ! NAMELIST group name
    !
    namelist /mars_dust_lifting_nml/                                  &
      & StressThreshold, KMHEfficiencyFactor, NewmanEfficiencyFactor                             
          !
          ! ǥեͤˤĤƤϽ³ "mars_dust_liftingk#MarsDustLiftingInit" 
          ! Υɤ򻲾ȤΤ. 
          !
          ! Refer to source codes in the initialization procedure
          ! "surface_flux_bulk#SurfFluxInit" for the default values. 
          !

    ! ¹ʸ ; Executable statement
    !

    if ( mars_dust_lifting_inited ) return

    ! ǥեͤ
    ! Default values settings
    !
    StressThreshold = 0.0225_DP! 22.5_DP
    KMHEfficiencyFactor = 0.1_DP
    NewmanEfficiencyFactor = 9.0_DP * 0.000001_DP

    ! NAMELIST ɤ߹
    ! NAMELIST is input
    !
    if ( trim(namelist_filename) /= '' ) then
      call FileOpen( unit_nml, &          ! (out)
        & namelist_filename, mode = 'r' ) ! (in)

      rewind( unit_nml )
      read( unit_nml, &                ! (in)
        & nml = mars_dust_lifting_nml, &  ! (out)
        & iostat = iostat_nml )        ! (out)
      close( unit_nml )

      call NmlutilMsg( iostat_nml, module_name ) ! (in)
    end if

    ! ҥȥǡϤΤΤؤѿϿ
    ! Register of variables for history data output
    !
    call HistoryAutoAddVariable( 'KMHF_d', &
      & (/ 'lon ', 'lat ', 'time' /), &
      & 'mars dust lifting rate  ', 'kg m-2 s-1' )
    call HistoryAutoAddVariable( 'NewmanF_d', &
      & (/ 'lon ', 'lat ', 'time' /), &
      & 'mars dust lifting rate  ', 'kg m-2 s-1' )
    !  ; Print
    !
    call MessageNotify( 'M', module_name, '----- Initialization Messages -----' )

    call MessageNotify( 'M', module_name, '  StressThreshold   = %f', d = (/ StressThreshold   /) )
    call MessageNotify( 'M', module_name, '  KMHEfficiencyFactor  = %f', d = (/ KMHEfficiencyFactor  /) )
    call MessageNotify( 'M', module_name, '  NewmanEfficiencyFactor  = %f', d = (/ NewmanEfficiencyFactor  /) )
    call MessageNotify( 'M', module_name, '-- version = %c', c1 = trim(version) )

    mars_dust_lifting_inited = .true.

  end subroutine MarsDustLiftingInit

  !-------------------------------------------------------------------

end module mars_dust_lifting
