Linux  R2.6.5-7.282-sn2 FORTRAN90/SX         Rev.360        Mon Oct  1 21:49:48 2012
FILE NAME: roughlen_Matthews.f90
PROGRAM NAME: roughlen_matthews
DIAGNOSTIC LIST

  LINE  LEVEL( NO.): DIAGNOSTIC MESSAGE

   184  opt  (1593): Loop nest collapsed into one loop.
   184  vec  (   1): Vectorized loop.
   192  opt  (1593): Loop nest collapsed into one loop.
   192  vec  (   1): Vectorized loop.
   207  opt  (1593): Loop nest collapsed into one loop.
   207  vec  (   1): Vectorized loop.
   272  opt  (1593): Loop nest collapsed into one loop.
   272  vec  (   1): Vectorized loop.
   277  vec  (   1): Vectorized loop.
   379  warn (   7): Characters in a line over this form limitation 132.
Linux  R2.6.5-7.282-sn2 FORTRAN90/SX         Rev.360        Mon Oct  1 21:49:48 2012
FILE NAME: roughlen_Matthews.f90
PROGRAM NAME: roughlen_matthews
TRANSFORMATION LIST

  LINE                   FORTRAN STATEMENT

     1  != Matthews Υǡ˴Ť٤
     2  !
     3  != set roughness length on land surface based on data by Matthews
     4  !
     5  ! Authors::   Yoshiyuki O. Takahashi
     6  ! Version::   $Id: roughlen_Matthews.f90,v 1.4 2011-08-05 01:02:28 yot Exp $
     7  ! Tag Name::  $Name: dcpam5-20120921 $
     8  ! Copyright:: Copyright (C) GFD Dennou Club, 2008-2009. All rights reserved.
     9  ! License::   See COPYRIGHT[link:../../../COPYRIGHT]
    10  !
    11  
    12  module roughlen_Matthews
    13  
    14    !== References
    15    !
    16    !  Matthews, E.,
    17    !    Global vegetation and land use: New high-resolution data bases for climate
    18    !    studies,
    19    !    J. Clim. Appl. Meteor., 22, 474, 1983.
    20    !
    21    !  Matthews, E.,
    22    !    Prescription of land-surface boundary conditions in GISS GCM II:
    23    !    A simple method based on fine-resolution data bases,
    24    !    NASA Technical memorandum #86096, 1984.
    25  
    26    ! ⥸塼 ; USE statements
    27    !
    28  
    29    ! ̷ѥ᥿
    30    ! Kind type parameter
    31    !
    32    use dc_types, only: DP, &      ! ټ¿. Double precision.
    33      &                 STRING     ! ʸ.       Strings.
    34  
    35    ! å
    36    ! Message output
    37    !
    38    use dc_message, only: MessageNotify
    39  
    40    ! ʸ ; Declaration statements
    41    !
    42    implicit none
    43    private
    44  
    45    ! ³
    46    ! Public procedure
    47    !
    48    public:: SetRoughLenLandMatthews
    49    public:: RoughLenMatthewsInit
    50  
    51    ! ѿ
    52    ! Public variables
    53    !
    54  
    55    ! ѿ
    56    ! Private variables
    57    !
    58    logical, save :: roughlen_matthews_inited = .false.
    59                                ! ե饰.
    60                                ! Initialization flag
    61  
    62    integer, parameter :: NLandUseType = 32
    63    integer, parameter :: NM84Element  =  9
    64    integer, parameter :: NM84Type     = 22
    65  
    66    integer, save      :: DataSurfCond2M84Type( 0:NLandUseType )
    67    real(DP)  , save      :: DataM84Weight( NM84Element, 0:NM84Type )
    68    real(DP)  , save      :: DataM84RL( NM84Element )
    69  
    70    real(DP)  , parameter :: z0_ice = 0.01d0
    71  
    72  
    73    !
    74    ! Matthews (1984) grouped UNESCO land use type into 22 groups, whose properties
    75    ! can be constructed by conbining properties of those of 9 land use 'elemental
    76    ! groups'. Note that a word 'elemental group' is not used by Matthews (1984).
    77    !
    78    !
    79    ! Table 2B of Matthews (1984), NASA Technical memorandum #86096
    80    !
    81    ! 9 'elemental group' is as follows.
    82    !   #1 desert
    83    !   #2 tundra
    84    !   #3 grassland
    85    !   #4 grassland with shrub cover
    86    !   #5 grassland with tree cover
    87    !   #6 deciduous forest
    88    !   #7 evergreen forest
    89    !   #8 rainforest
    90    !   #9 ice
    91    !
    92    !     %1   %2   %3   %4   %5   %6   %7   %8   %9
    93    data DataM84Weight / &
    94           0,   0,   0,   0,   0,   0,   0,   0,   0, & !  0, This is not included in Matthews papers.
    95           0,   0,   0,   0,   0,   0,   0, 100,   0, & !  1
    96           0,   0,  25,   0,   0,   0,  75,   0,   0, & !  2
    97          40,   0,   0,   0,   0,   0,  60,   0,   0, & !  3
    98           0,   0,   0,   0,   0,   0, 100,   0,   0, & !  4
    99           0,   0,  25,   0,   0,  75,   0,   0,   0, & !  5
   100           0,   0,   0,   0,   0, 100,   0,   0,   0, & !  6
   101          15,   0,   0,   0,   0,  85,   0,   0,   0, & !  7
   102          85,   0,   0,   0,   0,  15,   0,   0,   0, & !  8
   103          35,   0,   0,   0,   0,   0,  65,   0,   0, & !  9
   104          25,   0,  25,   0,   0,   0,  50,   0,   0, & ! 10
   105          35,   0,   0,   0,   0,  65,   0,   0,   0, & ! 11
   106          30,   0,   0,   0,   0,  70,   0,   0,   0, & ! 12
   107           0,   0,  50,   0,   0,  50,   0,   0,   0, & ! 13
   108          10,   0,  80,   0,   0,   0,  10,   0,   0, & ! 14
   109          10,   0,  80,   0,   0,  10,   0,   0,   0, & ! 15
   110           0, 100,   0,   0,   0,   0,   0,   0,   0, & ! 16
   111           0,   0,   0,   0, 100,   0,   0,   0,   0, & ! 17
   112           0,   0,   0, 100,   0,   0,   0,   0,   0, & ! 18
   113           0,   0, 100,   0,   0,   0,   0,   0,   0, & ! 19
   114         100,   0,   0,   0,   0,   0,   0,   0,   0, & ! 20
   115           0,   0,   0,   0,   0,   0,   0,   0, 100, & ! 21 ice
   116          30,   0,   0,   0,   0,   0,  70,   0,   0  & ! 22
   117          /
   118  
   119    !
   120    ! roughness length
   121    ! Table 3 of Matthews (1984). unit is cm.
   122    ! A value of roughness length for ice is not included in Matthews (1984).
   123    !
   124    !     %1     %2     %3     %4     %5     %6     %7     %8     %9
   125    data DataM84RL / &
   126         0.5,   0.5,   1.0,   1.0,   1.8,  32.0, 100.0, 200.0,   0 &
   127         /
   128  
   129    character(*), parameter:: module_name = 'roughlen_Matthews'
   130                                ! ⥸塼̾.
   131                                ! Module name
   132    character(*), parameter:: version = &
   133      & '$Name: dcpam5-20120921 $' // &
   134      & '$Id: roughlen_Matthews.f90,v 1.4 2011-08-05 01:02:28 yot Exp $'
   135                                ! ⥸塼ΥС
   136                                ! Module version
   137  
   138  contains
   139  
   140    !--------------------------------------------------------------------------------------
   141  
   142    subroutine SetRoughLenLandMatthews( xy_SurfCond, xy_SurfRoughLen )
   143  
   144      ! ⥸塼 ; USE statements
   145      !
   146  
   147      ! ʻ
   148      ! Grid points settings
   149      !
   150      use gridset, only: imax, & ! ٳʻ.
   151                                 ! Number of grid points in longitude
   152        &                jmax, & ! ٳʻ.
   153                                 ! Number of grid points in latitude
   154        &                kmax    ! ľؿ.
   155                                 ! Number of vertical level
   156  
   157      ! ʸ ; Declaration statements
   158      !
   159      integer , intent(in   ) :: xy_SurfCond    ( 0:imax-1, 1:jmax )
   160      real(DP), intent(inout) :: xy_SurfRoughLen( 0:imax-1, 1:jmax )
   161  
   162  
   163      ! ѿ
   164      ! Work variables
   165      !
   166      integer:: i               ! ˲ DO 롼Ѻѿ
   167                                ! Work variables for DO loop in longitude
   168      integer:: j               ! ˲ DO 롼Ѻѿ
   169                                ! Work variables for DO loop in latitude
   170      integer:: l
   171  
   172  
   173      ! ǧ
   174      ! Initialization check
   175      !
   176      if ( .not. roughlen_matthews_inited ) then
   177        call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   178      end if
   179  
   180  
   181      !
   182      !
   183      !
   184      do j = 1, jmax
   185        do i = 0, imax-1
   186          if( xy_SurfCond( i, j ) >= 1 ) then
   187            xy_SurfRoughLen( i, j ) = 0.0d0
   188          end if
   189        end do
   190      end do
     .  !CDIR NODEP                                                             
     .  !CDIR NOASSUME                                                          
     .        do j = 1, jmax*imax                                               
     .           if (xy_surfcond(j-1,1) .ge. 1) then                            
     .              xy_surfroughlen(j-1,1) = 0.0000000000000000e+000            
     .           endif                                                          
     .        end do                                                            
   191      do l = 1, NM84Element
   192        do j = 1, jmax
   193          do i = 0, imax-1
   194            if( xy_SurfCond( i, j ) >= 1 ) then
   195              xy_SurfRoughLen( i, j ) = xy_SurfRoughLen( i, j ) &
   196                + DataM84RL( l ) &
   197                * DataM84Weight( l, DataSurfCond2M84Type( xy_SurfCond( i, j ) ) )
   198            end if
   199          end do
   200        end do
     .  !CDIR NODEP                                                             
     .  !CDIR NOASSUME                                                          
     .        do j = 1, jmax*imax                                               
     .           if (xy_surfcond(j-1,1) .ge. 1) then                            
     .              xy_surfroughlen(j-1,1) = xy_surfroughlen(j-1,1) + datam84rl(
     .       1         l)*datam84weight(l,datasurfcond2m84type(xy_surfcond(j-1,1
     .       2         )))                                                      
     .           endif                                                          
     .        end do                                                            
   201      end do
   202  
   203  
   204      !
   205      ! ice
   206      !
   207      do j = 1, jmax
   208        do i = 0, imax-1
   209  
   210          if( xy_SurfCond( i, j ) == 31 ) then
   211            xy_SurfRoughLen( i, j ) = z0_ice
   212          end if
   213  
   214        end do
   215      end do
     .  !CDIR NODEP                                                             
     .  !CDIR NOASSUME                                                          
     .        do j = 1, jmax*imax                                               
     .           if (xy_surfcond(j-1,1) .eq. 31) then                           
     .              xy_surfroughlen(j-1,1) = 1.00000000000000e-002              
     .           endif                                                          
     .        end do                                                            
   216  
   217  
   218    end subroutine SetRoughLenLandMatthews
   219  
   220    !--------------------------------------------------------------------------------------
   221  
   222    subroutine RoughLenMatthewsInit
   223  
   224      ! ѿ
   225      ! Work variables
   226      !
   227      integer:: l
   228      integer:: m
   229  
   230  
   231      if ( roughlen_matthews_inited ) return
   232  
   233  
   234      !
   235      ! This table/list is created by using Table 4 of Matthews (1983) and Table 1 of
   236      ! Matthews (1984).
   237      !
   238      DataSurfCond2M84Type(  0 ) =  0 ! ocean, This is not included in Matthews papers.
   239      DataSurfCond2M84Type(  1 ) =  1
   240      DataSurfCond2M84Type(  2 ) =  1
   241      DataSurfCond2M84Type(  3 ) =  1
   242      DataSurfCond2M84Type(  4 ) =  1
   243      DataSurfCond2M84Type(  5 ) =  2
   244      DataSurfCond2M84Type(  6 ) =  3
   245      DataSurfCond2M84Type(  7 ) =  4
   246      DataSurfCond2M84Type(  8 ) =  4
   247      DataSurfCond2M84Type(  9 ) =  5
   248      DataSurfCond2M84Type( 10 ) =  6 ! A
   249      DataSurfCond2M84Type( 11 ) =  6 ! B, This may be 7. I cannot identify.
   250      DataSurfCond2M84Type( 12 ) =  8 ! C
   251      DataSurfCond2M84Type( 13 ) =  9 ! D
   252      DataSurfCond2M84Type( 14 ) = 10 ! E
   253      DataSurfCond2M84Type( 15 ) = 11 ! F
   254      DataSurfCond2M84Type( 16 ) = 12 ! G
   255      DataSurfCond2M84Type( 17 ) = 14 ! H
   256      DataSurfCond2M84Type( 18 ) = 14 ! I
   257      DataSurfCond2M84Type( 19 ) = 15 ! J
   258      DataSurfCond2M84Type( 20 ) = 15 ! K
   259      DataSurfCond2M84Type( 21 ) =  8 ! L
   260      DataSurfCond2M84Type( 22 ) = 16 ! M
   261      DataSurfCond2M84Type( 23 ) = 17 ! N
   262      DataSurfCond2M84Type( 24 ) = 17 ! O
   263      DataSurfCond2M84Type( 25 ) = 17 ! P
   264      DataSurfCond2M84Type( 26 ) = 19 ! Q
   265      DataSurfCond2M84Type( 27 ) = 19 ! R
   266      DataSurfCond2M84Type( 28 ) = 19 ! S
   267      DataSurfCond2M84Type( 29 ) = 19 ! T
   268      DataSurfCond2M84Type( 30 ) = 20 ! U
   269      DataSurfCond2M84Type( 31 ) = 21 ! V ice
   270      DataSurfCond2M84Type( 32 ) = 19 ! W
   271  
   272      do m = 0, NM84Type
   273        do l = 1, NM84Element
   274          DataM84Weight( l, m ) = DataM84Weight( l, m ) * 1.0d-2
   275        end do
   276      end do
     .  !CDIR NODEP                                                             
     .  !CDIR NOASSUME                                                          
     .        do m = 1, 207                                                     
     .           datam84weight(m,0) = datam84weight(m,0)*1.00000000000000e-002  
     .        end do                                                            
   277      do l = 1, NM84Element
   278        DataM84RL( l ) = DataM84RL( l ) * 1.0d-2
   279      end do
   280  
   281      roughlen_matthews_inited = .true.
   282  
   283    end subroutine RoughLenMatthewsInit
   284  
   285    !--------------------------------------------------------------------------------------
   286  
   287  !!$
   288  !!$  subroutine setroughnesslength_ocean( im, jm, km, gzs, gsfcindex, &
   289  !!$    gu, gv, gt, galt, gpt, gpth, gz0_out )
   290  !!$
   291  !!$    use vtype_module
   292  !!$    use const_module, only : grav
   293  !!$    use vdiff_module
   294  !!$
   295  !!$    implicit none
   296  !!$
   297  !!$    integer(i4b), intent(in   ) :: im, jm, km
   298  !!$    real(dp)    , intent(in   ) :: gzs( im, jm )
   299  !!$    integer(i4b), intent(in   ) :: gsfcindex( im, jm )
   300  !!$    real(dp)    , intent(in   ) :: gu( im, jm, km ), gv( im, jm, km ), gt( im, jm, km ), galt( im, jm, km ), gpt( im, jm, km ), gpth( im, jm, km+1 )
   301  !!$    real(dp)    , intent(inout) :: gz0_out( im, jm )
   302  !!$
   303  !!$    !
   304  !!$    ! local variables
   305  !!$    !
   306  !!$    real(dp)    , parameter :: gz0_init = 0.01d0
   307  !!$
   308  !!$    real(dp)                :: c_charnock
   309  !!$
   310  !!$    integer(i4b)            :: iitr, nitr
   311  !!$    real(dp)                :: cd( im, jm ), ch( im, jm ), fvsq( im, jm )
   312  !!$
   313  !!$    real(dp)                :: gz0( im, jm )
   314  !!$
   315  !!$    integer(i4b)            :: i, j
   316  !!$
   317  !!$
   318  !!$    !
   319  !!$    ! initial value for interation
   320  !!$    !
   321  !!$    do j = 1, jm
   322  !!$      do i = 1, im
   323  !!$        gz0( i, j ) = gz0_init
   324  !!$      end do
   325  !!$    end do
   326  !!$
   327  !!$    nitr = 10
   328  !!$    do iitr = 1, nitr
   329  !!$
   330  !!$      call vdiff_dragcoef_core( im, jm, km, gzs, &
   331  !!$        gz0, &
   332  !!$        gu, gv, gt, galt, gpt, gpth, cd, ch )
   333  !!$
   334  !!$      write( 6, * ) gsfcindex( 1, jm/2 ), iitr, gz0( 1, jm/2 ), cd( 1, jm/2 )
   335  !!$
   336  !!$
   337  !!$      do j = 1, jm
   338  !!$        do i = 1, im
   339  !!$          fvsq( i, j ) = ( cd( i, j ) + 1.0d-6 )&
   340  !!$            * ( gu( i, j, km )**2 + gv( i, j, km )**2 + 1.0d-3 )
   341  !!$        end do
   342  !!$      end do
   343  !!$
   344  !!$      !
   345  !!$      ! Krishnamurti?
   346  !!$!         c_charnock = 0.04d0
   347  !!$      !
   348  !!$      ! Wu (1967), technical report
   349  !!$      ! http://oai.dtic.mil/oai/oai?&verb=getRecord&metadataPrefix=html&identifier=AD0672750
   350  !!$!         c_charnock = 0.0156d0
   351  !!$      ! Wu (1980), JPO
   352  !!$      c_charnock = 0.0185d0
   353  !!$      do j = 1, jm
   354  !!$        do i = 1, im
   355  !!$          if( gsfcindex( i, j ) .le. 0 ) then
   356  !!$            ! Ocean
   357  !!$            ! Charnock, 1955
   358  !!$            gz0( i, j ) = c_charnock * fvsq( i, j ) / grav
   359  !!$          end if
   360  !!$        end do
   361  !!$      end do
   362  !!$
   363  !!$    end do
   364  !!$
   365  !!$
   366  !!$    do j = 1, jm
   367  !!$      do i = 1, im
   368  !!$
   369  !!$        if( gsfcindex( i, j ) .le. 0 ) then
   370  !!$          gz0_out( i, j ) = gz0( i, j )
   371  !!$        end if
   372  !!$
   373  !!$      end do
   374  !!$    end do
   375  !!$
   376  !!$
   377  !!$  end subroutine setroughnesslength_ocean
   378  
   379  end module roughlen_Matthews
Linux  R2.6.5-7.282-sn2 FORTRAN90/SX         Rev.360        Mon Oct  1 21:49:48 2012
FILE NAME: roughlen_Matthews.f90
PROGRAM NAME: roughlen_matthews
FORMAT LIST

  LINE    LOOP     FORTRAN STATEMENT

     1:            != Matthews Υǡ˴Ť٤
     2:            !
     3:            != set roughness length on land surface based on data by Matthews
     4:            !
     5:            ! Authors::   Yoshiyuki O. Takahashi
     6:            ! Version::   $Id: roughlen_Matthews.f90,v 1.4 2011-08-05 01:02:28 yot Exp $
     7:            ! Tag Name::  $Name: dcpam5-20120921 $
     8:            ! Copyright:: Copyright (C) GFD Dennou Club, 2008-2009. All rights reserved.
     9:            ! License::   See COPYRIGHT[link:../../../COPYRIGHT]
    10:            !
    11:            
    12:            module roughlen_Matthews
    13:            
    14:              !== References
    15:              !
    16:              !  Matthews, E.,
    17:              !    Global vegetation and land use: New high-resolution data bases for climate 
    18:              !    studies, 
    19:              !    J. Clim. Appl. Meteor., 22, 474, 1983. 
    20:              !
    21:              !  Matthews, E., 
    22:              !    Prescription of land-surface boundary conditions in GISS GCM II:
    23:              !    A simple method based on fine-resolution data bases, 
    24:              !    NASA Technical memorandum #86096, 1984.
    25:            
    26:              ! ⥸塼 ; USE statements
    27:              !
    28:            
    29:              ! ̷ѥ᥿
    30:              ! Kind type parameter
    31:              !
    32:              use dc_types, only: DP, &      ! ټ¿. Double precision.
    33:                &                 STRING     ! ʸ.       Strings.
    34:            
    35:              ! å
    36:              ! Message output
    37:              !
    38:              use dc_message, only: MessageNotify
    39:            
    40:              ! ʸ ; Declaration statements
    41:              !
    42:              implicit none
    43:              private
    44:            
    45:              ! ³
    46:              ! Public procedure
    47:              !
    48:              public:: SetRoughLenLandMatthews
    49:              public:: RoughLenMatthewsInit
    50:            
    51:              ! ѿ
    52:              ! Public variables
    53:              !
    54:            
    55:              ! ѿ
    56:              ! Private variables
    57:              !
    58:              logical, save :: roughlen_matthews_inited = .false.
    59:                                          ! ե饰.
    60:                                          ! Initialization flag
    61:            
    62:              integer, parameter :: NLandUseType = 32
    63:              integer, parameter :: NM84Element  =  9
    64:              integer, parameter :: NM84Type     = 22
    65:            
    66:              integer, save      :: DataSurfCond2M84Type( 0:NLandUseType )
    67:              real(DP)  , save      :: DataM84Weight( NM84Element, 0:NM84Type )
    68:              real(DP)  , save      :: DataM84RL( NM84Element )
    69:            
    70:              real(DP)  , parameter :: z0_ice = 0.01d0
    71:            
    72:            
    73:              !
    74:              ! Matthews (1984) grouped UNESCO land use type into 22 groups, whose properties
    75:              ! can be constructed by conbining properties of those of 9 land use 'elemental
    76:              ! groups'. Note that a word 'elemental group' is not used by Matthews (1984).
    77:              !
    78:              !
    79:              ! Table 2B of Matthews (1984), NASA Technical memorandum #86096
    80:              !
    81:              ! 9 'elemental group' is as follows.
    82:              !   #1 desert
    83:              !   #2 tundra
    84:              !   #3 grassland
    85:              !   #4 grassland with shrub cover
    86:              !   #5 grassland with tree cover
    87:              !   #6 deciduous forest
    88:              !   #7 evergreen forest
    89:              !   #8 rainforest
    90:              !   #9 ice
    91:              !
    92:              !     %1   %2   %3   %4   %5   %6   %7   %8   %9
    93:              data DataM84Weight / &
    94:                     0,   0,   0,   0,   0,   0,   0,   0,   0, & !  0, This is not included in Matthews papers.
    95:                     0,   0,   0,   0,   0,   0,   0, 100,   0, & !  1
    96:                     0,   0,  25,   0,   0,   0,  75,   0,   0, & !  2
    97:                    40,   0,   0,   0,   0,   0,  60,   0,   0, & !  3
    98:                     0,   0,   0,   0,   0,   0, 100,   0,   0, & !  4
    99:                     0,   0,  25,   0,   0,  75,   0,   0,   0, & !  5
   100:                     0,   0,   0,   0,   0, 100,   0,   0,   0, & !  6
   101:                    15,   0,   0,   0,   0,  85,   0,   0,   0, & !  7
   102:                    85,   0,   0,   0,   0,  15,   0,   0,   0, & !  8
   103:                    35,   0,   0,   0,   0,   0,  65,   0,   0, & !  9
   104:                    25,   0,  25,   0,   0,   0,  50,   0,   0, & ! 10
   105:                    35,   0,   0,   0,   0,  65,   0,   0,   0, & ! 11
   106:                    30,   0,   0,   0,   0,  70,   0,   0,   0, & ! 12
   107:                     0,   0,  50,   0,   0,  50,   0,   0,   0, & ! 13
   108:                    10,   0,  80,   0,   0,   0,  10,   0,   0, & ! 14
   109:                    10,   0,  80,   0,   0,  10,   0,   0,   0, & ! 15
   110:                     0, 100,   0,   0,   0,   0,   0,   0,   0, & ! 16
   111:                     0,   0,   0,   0, 100,   0,   0,   0,   0, & ! 17
   112:                     0,   0,   0, 100,   0,   0,   0,   0,   0, & ! 18
   113:                     0,   0, 100,   0,   0,   0,   0,   0,   0, & ! 19
   114:                   100,   0,   0,   0,   0,   0,   0,   0,   0, & ! 20
   115:                     0,   0,   0,   0,   0,   0,   0,   0, 100, & ! 21 ice
   116:                    30,   0,   0,   0,   0,   0,  70,   0,   0  & ! 22
   117:                    /
   118:            
   119:              !
   120:              ! roughness length
   121:              ! Table 3 of Matthews (1984). unit is cm.
   122:              ! A value of roughness length for ice is not included in Matthews (1984).
   123:              !
   124:              !     %1     %2     %3     %4     %5     %6     %7     %8     %9
   125:              data DataM84RL / &
   126:                   0.5,   0.5,   1.0,   1.0,   1.8,  32.0, 100.0, 200.0,   0 &
   127:                   /
   128:            
   129:              character(*), parameter:: module_name = 'roughlen_Matthews'
   130:                                          ! ⥸塼̾. 
   131:                                          ! Module name
   132:              character(*), parameter:: version = &
   133:                & '$Name: dcpam5-20120921 $' // &
   134:                & '$Id: roughlen_Matthews.f90,v 1.4 2011-08-05 01:02:28 yot Exp $'
   135:                                          ! ⥸塼ΥС
   136:                                          ! Module version
   137:            
   138:            contains
   139:            
   140:              !--------------------------------------------------------------------------------------
   141:            
   142:              subroutine SetRoughLenLandMatthews( xy_SurfCond, xy_SurfRoughLen )
   143:            
   144:                ! ⥸塼 ; USE statements
   145:                !
   146:            
   147:                ! ʻ
   148:                ! Grid points settings
   149:                !
   150:                use gridset, only: imax, & ! ٳʻ.
   151:                                           ! Number of grid points in longitude
   152:                  &                jmax, & ! ٳʻ.
   153:                                           ! Number of grid points in latitude
   154:                  &                kmax    ! ľؿ.
   155:                                           ! Number of vertical level
   156:            
   157:                ! ʸ ; Declaration statements
   158:                !
   159:                integer , intent(in   ) :: xy_SurfCond    ( 0:imax-1, 1:jmax )
   160:                real(DP), intent(inout) :: xy_SurfRoughLen( 0:imax-1, 1:jmax )
   161:            
   162:            
   163:                ! ѿ
   164:                ! Work variables
   165:                !
   166:                integer:: i               ! ˲ DO 롼Ѻѿ
   167:                                          ! Work variables for DO loop in longitude
   168:                integer:: j               ! ˲ DO 롼Ѻѿ
   169:                                          ! Work variables for DO loop in latitude
   170:                integer:: l
   171:            
   172:            
   173:                ! ǧ
   174:                ! Initialization check
   175:                !
   176:                if ( .not. roughlen_matthews_inited ) then
   177:                  call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
   178:                end if
   179:            
   180:            
   181:                !
   182:                !
   183:                !
   184: W------>       do j = 1, jmax
   185: |*----->         do i = 0, imax-1
   186: ||                 if( xy_SurfCond( i, j ) >= 1 ) then
   187: ||                   xy_SurfRoughLen( i, j ) = 0.0d0
   188: ||                 end if
   189: |*-----          end do
   190: W------        end do
   191: +------>       do l = 1, NM84Element
   192: |W----->         do j = 1, jmax
   193: ||*---->           do i = 0, imax-1
   194: |||                  if( xy_SurfCond( i, j ) >= 1 ) then
   195: |||                    xy_SurfRoughLen( i, j ) = xy_SurfRoughLen( i, j ) &
   196: |||                      + DataM84RL( l ) &
   197: |||                      * DataM84Weight( l, DataSurfCond2M84Type( xy_SurfCond( i, j ) ) )
   198: |||                  end if
   199: ||*----            end do
   200: |W-----          end do
   201: +------        end do
   202:            
   203:            
   204:                !
   205:                ! ice
   206:                !
   207: W------>       do j = 1, jmax
   208: |*----->         do i = 0, imax-1
   209: ||         
   210: ||                 if( xy_SurfCond( i, j ) == 31 ) then
   211: ||                   xy_SurfRoughLen( i, j ) = z0_ice
   212: ||                 end if
   213: ||         
   214: |*-----          end do
   215: W------        end do
   216:            
   217:            
   218:              end subroutine SetRoughLenLandMatthews
   219:            
   220:              !--------------------------------------------------------------------------------------
   221:            
   222:              subroutine RoughLenMatthewsInit
   223:            
   224:                ! ѿ
   225:                ! Work variables
   226:                !
   227:                integer:: l
   228:                integer:: m
   229:            
   230:            
   231:                if ( roughlen_matthews_inited ) return
   232:            
   233:            
   234:                !
   235:                ! This table/list is created by using Table 4 of Matthews (1983) and Table 1 of 
   236:                ! Matthews (1984).
   237:                !
   238:                DataSurfCond2M84Type(  0 ) =  0 ! ocean, This is not included in Matthews papers.
   239:                DataSurfCond2M84Type(  1 ) =  1
   240:                DataSurfCond2M84Type(  2 ) =  1
   241:                DataSurfCond2M84Type(  3 ) =  1
   242:                DataSurfCond2M84Type(  4 ) =  1
   243:                DataSurfCond2M84Type(  5 ) =  2
   244:                DataSurfCond2M84Type(  6 ) =  3
   245:                DataSurfCond2M84Type(  7 ) =  4
   246:                DataSurfCond2M84Type(  8 ) =  4
   247:                DataSurfCond2M84Type(  9 ) =  5
   248:                DataSurfCond2M84Type( 10 ) =  6 ! A
   249:                DataSurfCond2M84Type( 11 ) =  6 ! B, This may be 7. I cannot identify.
   250:                DataSurfCond2M84Type( 12 ) =  8 ! C
   251:                DataSurfCond2M84Type( 13 ) =  9 ! D
   252:                DataSurfCond2M84Type( 14 ) = 10 ! E
   253:                DataSurfCond2M84Type( 15 ) = 11 ! F
   254:                DataSurfCond2M84Type( 16 ) = 12 ! G
   255:                DataSurfCond2M84Type( 17 ) = 14 ! H
   256:                DataSurfCond2M84Type( 18 ) = 14 ! I
   257:                DataSurfCond2M84Type( 19 ) = 15 ! J
   258:                DataSurfCond2M84Type( 20 ) = 15 ! K
   259:                DataSurfCond2M84Type( 21 ) =  8 ! L
   260:                DataSurfCond2M84Type( 22 ) = 16 ! M
   261:                DataSurfCond2M84Type( 23 ) = 17 ! N
   262:                DataSurfCond2M84Type( 24 ) = 17 ! O
   263:                DataSurfCond2M84Type( 25 ) = 17 ! P
   264:                DataSurfCond2M84Type( 26 ) = 19 ! Q
   265:                DataSurfCond2M84Type( 27 ) = 19 ! R
   266:                DataSurfCond2M84Type( 28 ) = 19 ! S
   267:                DataSurfCond2M84Type( 29 ) = 19 ! T
   268:                DataSurfCond2M84Type( 30 ) = 20 ! U
   269:                DataSurfCond2M84Type( 31 ) = 21 ! V ice
   270:                DataSurfCond2M84Type( 32 ) = 19 ! W
   271:            
   272: W------>       do m = 0, NM84Type
   273: |*----->         do l = 1, NM84Element
   274: ||                 DataM84Weight( l, m ) = DataM84Weight( l, m ) * 1.0d-2
   275: |*-----          end do
   276: W------        end do
   277: V------>       do l = 1, NM84Element
   278: |                DataM84RL( l ) = DataM84RL( l ) * 1.0d-2
   279: V------        end do
   280:            
   281:                roughlen_matthews_inited = .true.
   282:            
   283:              end subroutine RoughLenMatthewsInit
   284:            
   285:              !--------------------------------------------------------------------------------------
   286:            
   287:            !!$
   288:            !!$  subroutine setroughnesslength_ocean( im, jm, km, gzs, gsfcindex, &
   289:            !!$    gu, gv, gt, galt, gpt, gpth, gz0_out )
   290:            !!$
   291:            !!$    use vtype_module
   292:            !!$    use const_module, only : grav
   293:            !!$    use vdiff_module
   294:            !!$
   295:            !!$    implicit none
   296:            !!$
   297:            !!$    integer(i4b), intent(in   ) :: im, jm, km
   298:            !!$    real(dp)    , intent(in   ) :: gzs( im, jm )
   299:            !!$    integer(i4b), intent(in   ) :: gsfcindex( im, jm )
   300:            !!$    real(dp)    , intent(in   ) :: gu( im, jm, km ), gv( im, jm, km ), gt( im, jm, km ), galt( im, jm, km ), gpt( im, jm, km ), gpth( im, jm, km+1 )
   301:            !!$    real(dp)    , intent(inout) :: gz0_out( im, jm )
   302:            !!$
   303:            !!$    !
   304:            !!$    ! local variables
   305:            !!$    !
   306:            !!$    real(dp)    , parameter :: gz0_init = 0.01d0
   307:            !!$
   308:            !!$    real(dp)                :: c_charnock
   309:            !!$
   310:            !!$    integer(i4b)            :: iitr, nitr
   311:            !!$    real(dp)                :: cd( im, jm ), ch( im, jm ), fvsq( im, jm )
   312:            !!$
   313:            !!$    real(dp)                :: gz0( im, jm )
   314:            !!$
   315:            !!$    integer(i4b)            :: i, j
   316:            !!$
   317:            !!$
   318:            !!$    !
   319:            !!$    ! initial value for interation
   320:            !!$    !
   321:            !!$    do j = 1, jm
   322:            !!$      do i = 1, im
   323:            !!$        gz0( i, j ) = gz0_init
   324:            !!$      end do
   325:            !!$    end do
   326:            !!$
   327:            !!$    nitr = 10
   328:            !!$    do iitr = 1, nitr
   329:            !!$
   330:            !!$      call vdiff_dragcoef_core( im, jm, km, gzs, &
   331:            !!$        gz0, &
   332:            !!$        gu, gv, gt, galt, gpt, gpth, cd, ch )
   333:            !!$
   334:            !!$      write( 6, * ) gsfcindex( 1, jm/2 ), iitr, gz0( 1, jm/2 ), cd( 1, jm/2 )
   335:            !!$
   336:            !!$
   337:            !!$      do j = 1, jm
   338:            !!$        do i = 1, im
   339:            !!$          fvsq( i, j ) = ( cd( i, j ) + 1.0d-6 )&
   340:            !!$            * ( gu( i, j, km )**2 + gv( i, j, km )**2 + 1.0d-3 )
   341:            !!$        end do
   342:            !!$      end do
   343:            !!$
   344:            !!$      !
   345:            !!$      ! Krishnamurti?
   346:            !!$!         c_charnock = 0.04d0
   347:            !!$      !
   348:            !!$      ! Wu (1967), technical report
   349:            !!$      ! http://oai.dtic.mil/oai/oai?&verb=getRecord&metadataPrefix=html&identifier=AD0672750
   350:            !!$!         c_charnock = 0.0156d0
   351:            !!$      ! Wu (1980), JPO
   352:            !!$      c_charnock = 0.0185d0
   353:            !!$      do j = 1, jm
   354:            !!$        do i = 1, im
   355:            !!$          if( gsfcindex( i, j ) .le. 0 ) then
   356:            !!$            ! Ocean
   357:            !!$            ! Charnock, 1955
   358:            !!$            gz0( i, j ) = c_charnock * fvsq( i, j ) / grav
   359:            !!$          end if
   360:            !!$        end do
   361:            !!$      end do
   362:            !!$
   363:            !!$    end do
   364:            !!$
   365:            !!$
   366:            !!$    do j = 1, jm
   367:            !!$      do i = 1, im
   368:            !!$
   369:            !!$        if( gsfcindex( i, j ) .le. 0 ) then
   370:            !!$          gz0_out( i, j ) = gz0( i, j )
   371:            !!$        end if
   372:            !!$
   373:            !!$      end do
   374:            !!$    end do
   375:            !!$
   376:            !!$
   377:            !!$  end subroutine setroughnesslength_ocean
   378:            
   379:            end module roughlen_Matthews
